(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



AutoGeneratedPackage->Automatic;


SetDirectory[NotebookDirectory[]];


Clear[\[Theta]];Clear[\[Phi]]; Clear[x]; Clear[y]; Clear[z];


MF::usage = "MF[mtx] returns the matrix form of the matrix <mtx>.

EX.1
MF[{{1,2},{3,4}}]
";
MF[a_]:=MatrixForm[a];


KP::usage = "K[A,B,C,...] returns the Kronecker product of the matrices A,B,C,...

EX.1  
KP[{{1,2},{3,4}},{{0,1},{-1,0}}]
MF[%]

EX.2
MF[KP[{{1,2},{3,4}},{{0,1},{-1,0}},{{1,0,0,1}}]]
";
KP[x__,y__]:=KroneckerProduct[x,y];


Dagger::usage= "Dagger[mtx] returns the adjoint matrix of mtx, that is its conjugate transpose.

EX.1
mtx = {{1,I},{3,4}}
Dagger[mtx]
";
Dagger[a_]:=Transpose[ComplexExpand[Conjugate[a]]];


IsHermitian::usage = "IsHermitian[mtx] returns 1 if mtx is Hermitian and 0 otherwise.

EX.1 
a = {{1,I},{3,4}}
IsHermitian[a]

EX.2
a = {{1,I},{3,4}}
b = a + Dagger[a]
IsHermitian[b]
";
IsHermitian[mtx_]:=If[mtx==Dagger[mtx],Return[1],Return[0]];


Prod::usage = "Prod[a,b] takes two square matrices a and b of same dimension and returns their Hilbert product Tr[Dagger[a].b] 

EX.1 
a = {{1,I},{3,4}}
b = {{2,3+I},{3+I,8}}
Prod[a,b]

EX.2 
a = {{1,I},{3,4}}
Prod[a,a]
";
Prod[a_,b_]:=Sum[Conjugate[a[[i]]].b[[i]],{i,Length[a]}];(*Tr[Dagger[a].b]*) ;


MyNorm::usage ="MyNorm[a] returns the norm of a square matrix a, where the norm is defined by: MyNorm[a] = Sqrt[Prod[a,a]].

EX.1
a = {{1,I},{3,4}}
MyNorm[a]
";
MyNorm[a_]:=Sqrt[Prod[a,a]];


normalize::usage = "normalize[a] takes a square matrix a and returns its normalized version a/ ||a||.

EX.1
a = {{1,I},{3,4}};
MyNorm[a]
X=normalize[a];
MyNorm[X]
";
normalize[mtx_]:=Block[{norm},
norm = Prod[mtx,mtx];
If[norm== 0,
Return[mtx];,
Return[(1/Sqrt[norm]) mtx];];
];


RandomMatrix::usage = "RandomMatrix returns a random normalized complex 8x8 matrix.

EX.1 
a = RandomMatrix
MyNorm[a]
";
RandomMatrix:=normalize[ParallelTable[RandomComplex[{-1-I,1+I}],{i,1,8},{j,1,8}]];


MatLog::usage = "MatLog[X] returns a matrix Y such that MatrixExp[Y]=X.

EX.1
a = {{1,0},{0,2E+3I}}
b = MatrixExp[a]
MF[FullSimplify[MatLog[b]]]

EX.2
MF[MatLog[{{1,0},{0,2E+3I}}]]
";

MatLog[X_]:=Block[{dim,eigenvectors,S,IS,mlog,eigenvalues},
dim = Dimensions[X,1][[1]];
{eigenvalues,eigenvectors}=Eigensystem[X];
S=Transpose[eigenvectors];
IS=Inverse[S];
mlog = Dot[S,DiagonalMatrix[Log[eigenvalues]],IS];
Return[mlog];
];


ascii2num::usage="ascii2num[x] converts an ascii string of the form x=0.123...e+02 to a form understood by Mathematica.

Ex.1 
ascii2num[''1.825e+01'']";
ascii2num[str_]:=Block[{strSplit,body,exponent},
If[str== "NaN",Return[0],
strSplit = StringSplit[str,"e"]; (*{body,exponent}*);
body = ToExpression[strSplit[[1]]];
exponent = ToExpression[strSplit[[2]]];
Return[Chop[body*10^exponent]];
];(*If*)
];


Sh::usage = "Sh[J,m] returns the spherical harmonic of rank J and order m, in the variables \[Theta] and \[Phi]. 

EX.1 
Sh[2,-1]
" ;
Sh[J_,m_]:=Return[SphericalHarmonicY[J,m,\[Theta],\[Phi]]];


ToStr::usage="ToStr[number,width,fill] returns the formatted <number> with a minimum string-length of <width>. Missing elements are filled with the <fill> character.

EX.1
ToStr[359,8,''0'']
";
ToStr[number_,width_,fill_]:=Block[{str=Characters[ToString[number]]},
For[n=width-Length[str],n>=1,n--,
PrependTo[str,fill];];
Return[StringJoin[str]];
];
ToStr[number_,width_]:=ToStr[number,width," "];


TensorProd::usage="TensorProd[T1,T2] returns the tensor product of T1 and T2 according to Clebsch-Gordan decomposition, where T1 is a list of (2*j1)+1 matrices in the order m=-j1, -j1+1,... j1-1,j1 and T2 is a list of (2*j2)+1 matrices in the order m=-j2, -j2+1,... j2-1,j2.
The output is the list TENSORS, where TENSOR[[k]] is a list of (2*jk)+1 matrices in the order m=-jk, -jk+1,... jk-1,jk for and |j1-j2|<= jk <= j1+j2.
IMPORTANT: The product is taken to be the matrix product, i.e. the tensors T1 and T2 should already be imbedded in the full system.
jmin=|j1-j2|.

EX.1
I2 = IdentityMatrix[2];

T1 = {KP[tensor[1,-1],I2],KP[tensor[1,-1],I2],KP[tensor[1,-1],I2]};
T2 = {KP[I2,tensor[1,-1]],KP[I2,tensor[1,-1]],KP[I2,tensor[1,-1]]};
TENSOR = TensorProd[T1,T2];

Dimensions[TENSOR]
Dimensions[TENSOR[[1]]] (*jk=|j1-j2|=0*)
Dimensions[TENSOR[[2]]]
Dimensions[TENSOR[[3]]] (*jk=j1+j2=2*)
";
TensorProd[TA_,TB_]:=Block[{dim,j1,j2,T1,T2,j,jmin,jmax,TENSORS,m,m1,m2,T},
dim = Dimensions[TA[[1]]][[1]];
j1=(Length[TA]-1)/2;
j2=(Length[TB]-1)/2;
(*Convert TA in T1 for intuitive labelling T1[j1,m1]*)
Table[T1[m1],{m1,-j1,j1}];
For[m1=-j1,m1<= j1,m1++,T1[m1]=TA[[m1+j1+1]];];
(*Convert TB in T2 for intuitive labelling T2[j2,m2]*)
Table[T2[m2],{m2,-j2,j2}];
For[m2=-j2,m2<= j2,m2++,T2[m2]=TB[[m2+j2+1]];];
(*Range of values for j*)
jmin = Abs[j1-j2];
jmax = j1+j2;
(*Initialization of the T[j,m] variables*)
Table[T[J,M],{J,jmin,jmax},{M,-J,J}];
(*Computation of T[j,m] variables*)
For[j=jmin,j<=jmax,j++,
For[m=-j,m<= j,m++,
T[j,m]=Table[0,{i,dim},{j,dim}];(*Initialization*)
For[m1=-j1,m1<= j1,m1++,
If[-j2<= (m-m1)<=j2,
T[j,m]+=ClebschGordan[{j1,m1},{j2,m-m1},{j,m}]*(T1[m1].T2[m-m1]);,
##&[]];(*IF*)
];(*For m1*)
];(*FOR M*)
];(*for J*)
TENSORS=Table[T[J,M],{J,jmin,jmax},{M,-J,J}];
Return[TENSORS];
(*
Denoting jmin := |j1-j2|,
> TENSORS[[K,L]] := T[j,m] where j = jmin-1+K and m = -j+L-1
and conversely 
> T[j,m]:=TENSORS[[K,L]]   where K = j-jmin+1 and L = m+j+1
The order of the tensors in TENSORS goes from |j1-j2|...j1+j2.
*)
];(*Block*)


KPTensorProd::usage="KPTensorProd[T1,T2] returns the tensor product of T1 and T2 according to Clebsch-Gordan decomposition, where T1 is a list of (2*j1)+1 matrices in the order m=-j1, -j1+1,... j1-1,j1 and T2 is a list of (2*j2)+1 matrices in the order m=-j2, -j2+1,... j2-1,j2.
The output is the list TENSORS, where TENSOR[[k]] is a list of (2*jk)+1 matrices in the order m=-jk, -jk+1,... jk-1,jk for and |j1-j2|<= jk <= j1+j2. 
IMPORTANT: The product is taken to be the Kronecker product, i.e. the tensors T1 and T2 are not considered to be imbedded in the full system.

EX.1
T1 = {tensor[1,-1],tensor[1,0],tensor[1,1]};
T2 = T1;
TENSOR = KPTensorProd[T1,T2];

Dimensions[TENSOR]
Dimensions[TENSOR[[1]]] (*jk=|j1-j2|=0*)
Dimensions[TENSOR[[2]]]
Dimensions[TENSOR[[3]]] (*jk=j1+j2=2*)
";
KPTensorProd[TA_,TB_]:=Block[{dim,j1,j2,T1,T2,j,jmin,jmax,TENSORS,m,m1,m2,T},
dim = Dimensions[TA[[1]]][[1]]* Dimensions[TB[[1]]][[1]];
j1=(Length[TA]-1)/2;
j2=(Length[TB]-1)/2;
(*Convert TA in T1 for intuitive labelling T1[j1,m1]*)
Table[T1[m1],{m1,-j1,j1}];
For[m1=-j1,m1<= j1,m1++,T1[m1]=TA[[m1+j1+1]];];
(*Convert TB in T2 for intuitive labelling T2[j2,m2]*)
Table[T2[m2],{m2,-j2,j2}];
For[m2=-j2,m2<= j2,m2++,T2[m2]=TB[[m2+j2+1]];];
(*Range of values for j*)
jmin = Abs[j1-j2];
jmax = j1+j2;
(*Initialization of the T[j,m] variables*)
Table[T[J,M],{J,jmin,jmax},{M,-J,J}];
(*Computation of T[j,m] variables*)
For[j=jmin,j<=jmax,j++,
For[m=-j,m<= j,m++,
T[j,m]=Table[0,{i,dim},{j,dim}];(*Initialization*)
For[m1=-j1,m1<= j1,m1++,
If[-j2<= (m-m1)<=j2,
T[j,m]+=ClebschGordan[{j1,m1},{j2,m-m1},{j,m}]*(KP[T1[m1],T2[m-m1]]);,
##&[]];(*IF*)
];(*For m1*)
];(*FOR M*)
];(*for J*)
TENSORS=Table[T[J,M],{J,jmin,jmax},{M,-J,J}];
Return[TENSORS];
(*
Denoting jmin := |j1-j2|,
> TENSORS[[K,L]] := T[j,m] where j = jmin-1+K and m = -j+L-1
and conversely 
> T[j,m]:=TENSORS[[K,L]]   where K = j-jmin+1 and L = m+j+1
The order of the tensors in TENSORS goes from |j1-j2|...j1+j2.
*)
];(*Block*)


IInorm::usage = "IInorm[...] returns the 8x8 cartesian matrices for a 3 spins 1/2 system normalized to Sqrt[2].
The notation depends on the number of spins involved in the Cartesian operator and is identical to the notation for  the IIone and II version of the Cartesian operators:
* One spin involved: IInorm[s,x], IInorm[s,y], IInorm[s,z] , IInorm[s,E], where s=1,2,3.
* Two spins involved: IInorm[s1,x,s2,x], IInorm[s1,x,s2,y], [...], IInorm[s1,z,s2,y], IInorm[s1,z,s2,z], where s1,s2=1,2,3 AND s1!=s2.
* Three spins involved in the spin order 1.2.3 : IInorm[x,x,x], IInorm[x,x,y] , [...], IInorm[z,z,y], IInorm[z,z,z].

EX.1
IInorm[1,x]
MyNorm[%]

EX.2
IInorm[2,x,3,y]
MyNorm[%]

EX.3
IInorm[x,y,z]
MyNorm[%]
";


II1[symbol1_]:= Block[ {dummy},
Switch[{symbol1},
{E},dummy={{1,0},{0,1}},
{x},dummy={{0,1/2},{1/2,0}},
{y},dummy= {{0,-I/2},{I/2,0}},
{z},dummy={{1/2,0},{0,-1/2}}];
Return[ dummy];];

(*Map[MyNorm[#]&,{IInorm[E],IInorm[x],IInorm[y],IInorm[z]}]*)


IInorm[symbol_]:= Block[ {N,dummy},
N = Sqrt[2];
dummy=KroneckerProduct[II1[symbol],II1[symbol],II1[symbol]];
Return[N*normalize[dummy]];]


IInorm[spin_,symbol_]:= Block[ {N,dummy},
N=Sqrt[2];
Switch[{spin},
{1},dummy=KroneckerProduct[II1[symbol],II1[E],II1[E]],
{2},dummy=KroneckerProduct[II1[E],II1[symbol],II1[E]],
{3},dummy=KroneckerProduct[II1[E],II1[E],II1[symbol]]];
Return[N*normalize[dummy]];]


IInorm[spin1_,symbol1_,spin2_,symbol2_]:=Return[2*IInorm[spin1,symbol1].IInorm[spin2,symbol2]];


IInorm[symbol1_,symbol2_,symbol3_]:=Return[4*IInorm[1,symbol1].IInorm[2,symbol2].IInorm[3,symbol3]];


IIone::usage="Returns the Cartesian operators which are normalized to one.
The notation depends on the number of spins involved in the operator and is identical to the notation for the IInorm and II operators:
* One spin involved: IIone[s,x], IIone[s,y], IIone[s,z] , IIone[s,E], where s=1,2,3.
* Two spins involved: IIone[s1,x,s2,x], IIone[s1,x,s2,y], [...], IIone[s1,z,s2,y], IIone[s1,z,s2,z], where s1,s2=1,2,3 AND s1!=s2.
* Three spins involved in the spin order 1.2.3 : IIone[x,x,x], IIone[x,x,y] , [...], IIone[z,z,y], IIone[z,z,z].

EX.1
Map[MyNorm[#]&,{IIone[1,x],IIone[1,y],IIone[1,z]}]
";
IIone[a__]:=normalize[IInorm[a]];


II::usage="II is the version of the Cartesian operators which an additional factors to be normalized.
* For instance, 1/2*II[1,E], II[k,x], 2*II[k,x,l,y] and 4*II[x,y,z] have norm the NMR norm \!\(\*SqrtBox[\(2\)]\).

EX.1.
MF[II[1,x,2,x]]
MF[2*II[1,x,2,x]]
MF[IInorm[1,x,2,x]]

EX.2.
Map[MyNorm[#]&,{1/2*II[1,E]}]
Map[MyNorm[#]&,{II[1,x],II[1,y],II[1,z]}]
Map[MyNorm[#]&,{2 II[1,x,2,x],2 II[1,x,2,y],2 II[1,x,2,z]}]
Map[MyNorm[#]&,{4 II[x,x,x],4 II[x,x,y],4 II[x,x,z]}]
";


II[symbol_]:=Block[{N},
N=2*Sqrt[2];
Return[N IIone[symbol]];
];

Map[MyNorm[#]&,{II[E],II[1,x],II[1,y],II[1,z]}]


II[spin_,symbol_]:=Block[{N},
N=Sqrt[2];
If[symbol== E,N=2*Sqrt[2];];
Return[N IIone[spin,symbol]];
];

Map[MyNorm[#]&,{II[E],II[1,x],II[1,y],II[1,z]}]


II[spin1_,symbol1_,spin2_,symbol2_]:=Return[II[spin1,symbol1].II[spin2,symbol2]];


II[symbol1_,symbol2_,symbol3_]:=Return[II[1,symbol1].II[2,symbol2].II[3,symbol3]];


Format[iIInorm[symbol_]]:=DisplayForm[SequenceForm["1/2I[",symbol,"]"]];

Format[iIInorm[spin_,symbol_]]:=DisplayForm[SubscriptBox["I",SequenceForm[spin,symbol]]];
Format[iIInorm[spin1_,symbol1_,spin2_,symbol2_]]:=DisplayForm[SequenceForm[SubscriptBox["2I",SequenceForm[spin1,symbol1]],SubscriptBox["I",SequenceForm[spin2,symbol2]]]];
Format[iIInorm[symbol1_,symbol2_,symbol3_]]:=DisplayForm[SequenceForm[SubscriptBox["4I",SequenceForm["1",symbol1]],SubscriptBox["I",SequenceForm["2",symbol2]],SubscriptBox["I",SequenceForm["3",symbol3]]]];

(*EX .1
iIInorm[3,x]
iIInorm[1,x,3,z]
iIInorm[z,z,z]
*)


Format[iII[symbol_]]:=DisplayForm[SequenceForm["I[",symbol,"]"]];

Format[iII[spin_,symbol_]]:=DisplayForm[SubscriptBox["I",SequenceForm[spin,symbol]]];

Format[iII[spin1_,symbol1_,spin2_,symbol2_]]:=DisplayForm[SequenceForm[SubscriptBox["I",SequenceForm[spin1,symbol1]],SubscriptBox["I",SequenceForm[spin2,symbol2]]]];

Format[iII[symbol1_,symbol2_,symbol3_]]:=DisplayForm[SequenceForm[SubscriptBox["I",SequenceForm["1",symbol1]],SubscriptBox["I",SequenceForm["2",symbol2]],SubscriptBox["I",SequenceForm["3",symbol3]]]];
(*EX .1
iII[x]
iII[3,E]
iII[1,x,3,z]
iII[z,z,z]
*)


tensor::usage="tensor[...] returns the k-linear tensors obtained recursively by taking the product of tensors for the (k-1)-spin-1/2 system with a 1-spin-1/2 system, using the Clebsch decomposition. 
These tensors are not normalised and are principally used as a starting point to define the Tprod and Tsym tensors.
Type OrderBasis[Tprod] to see some possible parameter combinations taken by the method tensor.
> tensor[j,m] where (j,m) = (0,0),(1,-1),(1,0) or (1,1), are 2\[Times]2 matrices and correspond to the tensor operators for a single spin-1/2 system.
> tensor[spin,j,m] where (j,m) = (0,0),(1,-1),(1,0) or (1,1) are 8\[Times]8 matrices and correspond to the linear tensor operators acting on the spin #<spin> in a 3-spin-1/2 system, where <spin>=1,2 or 3.
> tensor[spin1,spin2,j,m] where j=0,1,2 and m\[Element]{-j,...,j} are 8\[Times]8 matrices corresponding to the bilinear operators acting on  <spin1> and <spin2> only. Here, <spini>\[Element]{1,2,3} and only the matrices for which <spin1> \[NotEqual] <spin2> are different of zero.
> tensor[1,2,3,j,m,parents] corresponds to the trilinear operators with rank j, order m and where <parent> corresponds to the parents in the product from which a tensor is obtained. For instance, tensor[1,2,3,3,-1,{2,1}] is a tensor of rank j=3, order m=-1, and obtained from the product tensor[1,2,2,m]\[CircleTimes]tensor[3,1,m] where the parents have rank j=2 and j=1. The accepted ranks of a tensor with parents {j1,j2} are j\[Element]{|j1-j2|,...,j1+j2}.

EX.1
MF[tensor[0,0]]
{MF[tensor[1,-1]],MF[tensor[1,0]],MF[tensor[1,1]]}

EX.2
{MF[tensor[3,1,-1]],MF[tensor[3,1,0]],MF[tensor[3,1,1]]}
{MF[tensor[1,1,0]],MF[tensor[2,1,0]],MF[tensor[3,1,0]]}

EX.3
{MF[tensor[1,2,2,-2]],MF[tensor[1,3,2,-2]],MF[tensor[2,3,2,-2]]}

EX.4
MF[tensor[1,2,3,3,-3,{2,1}]]
";


tensor[J_,m_]:=Block[ {dummy},
Switch[{J,m},{1,1},dummy={{0,-1},{0,0}},{1,0},dummy=Sqrt[1/2] {{1,0},{0,-1}},{1,-1},dummy={{0,0},{1,0}},{0,0},dummy=Sqrt[1/2] {{1,0},{0,1}}];
Return[ dummy];]


tensor2[J_,m_]:=Block[{j1,j2,T1,T2,jmin,TENSOR,K,L},
j1 =1 ;
j2 = 1;
T1 =  Table[KroneckerProduct[tensor[j1,m1],tensor[0,0]],{m1,-j1,j1}];
T2 =  Table[KroneckerProduct[tensor[0,0],tensor[j2,m2]],{m2,-j2,j2}];
jmin = Abs[j1-j2]; 
TENSOR =  TensorProd[T1,T2];
(* T[j,m]:=TENSORS[[K,L]]   where K = j-jmin+1 and L = m+j+1*)
K = J-jmin+1;
L = m+J+1;
Return[TENSOR[[K,L]]];]


tensor[spin_,J_,m_]:= Block[ {dummy},
Switch[{spin},
{0},dummy=KroneckerProduct[tensor[0,0],tensor[0,0],tensor[0,0]],
{1},dummy=KroneckerProduct[tensor[J,m],tensor[0,0],tensor[0,0]],
{2},dummy=KroneckerProduct[tensor[0,0],tensor[J,m],tensor[0,0]],
{3},dummy=KroneckerProduct[tensor[0,0],tensor[0,0],tensor[J,m]]];
Return[dummy];]	


tensor[spin1_,spin2_,J_,m_]:=Block[{j1,j2,T1,T2,jmin,TENSOR,K,L},
j1 =1 ;
j2 = 1;
T1 =  Table[tensor[spin1,j1,m1],{m1,-j1,j1}];
T2 =  Table[tensor[spin2,j2,m2],{m2,-j2,j2}];
jmin = Abs[j1-j2]; 
TENSOR =  TensorProd[T1,T2];
(* T[j,m]:=TENSORS[[K,L]]   where K = j-jmin+1 and L = m+j+1*)
K = J-jmin+1;
L = m+J+1;
Return[TENSOR[[K,L]]];]


tensor[spin1_,spin2_,spin3_,J_,m_,idx_]:=Block[{j12,j3,T12,m12,T3,m3,jmin,TENSOR,K,L},
(*idx = {0,1},{1,1} or {2,1} are the parents*)
j12 =idx[[1]];(* j12 \in 0,1,2*)
j3 =idx[[2]];(* j3 = 1 *)

T12 = Table[tensor[spin1,spin2,j12,m12],{m12,-j12,j12}];
T3   = Table[tensor[spin3,j3,m3],{m3,-j3,j3}];
jmin = Abs[j12-j3]; 
TENSOR =  TensorProd[T12,T3];
(* T[j,m]:=TENSORS[[K,L]]   where K = j-jmin+1 and L = m+j+1*)
K = J-jmin+1;
L = m+J+1;
Return[TENSOR[[K,L]]];];

(*tensor[1,2,3,1,-1,{1,1}]*)


Tprod::usage="Tprod[...] corresponds to the tensor basis operators of a 3-spin-1/2 system, obtained recursively by taking the product of tensors for the (k-1)-spin-1/2 system with a 1-spin-1/2 system, using the Clebsch decomposition.
These operators are 8\[Times]8 matrices, are normalized and are defined such that the operators of order m=0 are Hermitian.
Type OrderBasis[Tprod] to see some possible parameter combinations taken by the method tensor.

> Tprod[0,0,0] is proportional to the identity operator.
> Tprod[spin,j,m] where (j,m) = (0,0),(1,-1),(1,0) or (1,1) correspond to the linear tensor operators acting on the spin #<spin> in a 3-spin-1/2 system, where <spin>=1,2 or 3. Tprod[spin,j,0] are Hermitian matrices.
> Tprod[spin1,spin2,j,m] where j=0,1,2 and m\[Element]{-j,...,j} correspond to the bilinear operators acting on  <spin1> and <spin2> only. Here, <spini>\[Element]{1,2,3} and only the matrices for which <spin1> \[NotEqual] <spin2> are different of zero. Tprod[spin1,spin2,j,0] are Hermitian matrices.
> Tprod [1,2,3,j,m,parents] corresponds to the trilinear operators with rank j, order m and where <parent> corresponds to the parents in the product from which a tensor is obtained. For instance, Tprod[1,2,3,3,-1,{2,1}] is a tensor of rank j=3, order m=-1, and obtained from the product tensor[1,2,2,m]\[CircleTimes]tensor[3,1,m] where the parents have rank j=2 and j=1. The accepted ranks of a tensor with parents {j1,j2} are j\[Element]{|j1-j2|,...,j1+j2}. Tprod[1,2,3,j,0,parents] are Hermitian matrices.

EX.1.
OrderBasis[Tprod]

EX.2
MF[Tprod[1,1,-1]]
MF[Tprod[1,2,2,0]]
MF[Tprod[1,2,3,3,-2,{2,1}]]

EX.3
IsHermitian[tensor[1,2,3,2,0,{2,1}]]
IsHermitian[Tprod[1,2,3,2,0,{2,1}]]
";


TPROD[x__]:=normalize[tensor[x]];


pfTprod[spin_,J_,m_]:=Return[1]; 
pfTprod[spin1_,spin2_,J_,m_]:=Switch[{J},{2},Return[1];,{1},Return[-I];,{0},Return[-1];]; 
pfTprod[spin1_,spin2_,spin3_, J_,m_,idx_]:=Switch[{J},
{0},Return[I];,
{1},Return[1];,
{2},Return[I];,
{3},Return[1];];


Tprod[x__]:=pfTprod[x]*TPROD[x];


Format[iTprod[J_,m_]]:=DisplayForm[SubscriptBox["T",SequenceForm["p,"J,",",m]]];
Format[iTprod[spin_,J_,m_]]:=DisplayForm[SubsuperscriptBox["T",SequenceForm["p,",J,",",m],SequenceForm["{",spin,"}"]]];
Format[iTprod[spin1_,spin2_,J_,m_]]:=DisplayForm[SubsuperscriptBox["T",SequenceForm["p,",J,",",m],SequenceForm["{",spin1,spin2,"}"]]];
(*Format[iTprod[spin1_,spin2_,spin3_,J_,m_,idx_]]:=DisplayForm[
SequenceForm[
SubsuperscriptBox["T",SequenceForm[J,",",m],SequenceForm["{",spin1,spin2,spin3 "}"]],"[",idx[[1]],",",idx[[2]],"]"]];*)
Format[iTprod[spin1_,spin2_,spin3_,J_,m_,idx_]]:=DisplayForm[
SequenceForm[
SubsuperscriptBox["T",SequenceForm["p",J,",",m],SequenceForm["{",spin1,spin2,spin3 "}"]],"[",idx,"]"]];
(*EX
iTprod[1,1]
iTprod[3,1,-1]
iTprod[1,3,2,-1]
iTprod[1,2,3,2,-2,{2,1}]
*)


Nb2Symb::usage = "Nb2Symb[n] converts the numbers 1,2 and 3 into the Cartesian axis variables according to: 1->x , 2->y , 3->z.

EX.1
Nb2Symb[1]
Nb2Symb[2]
Nb2Symb[3]
";

Nb2Symb[nb_]:=Switch[{nb},{1},Return[x],{2},Return[y],{3},Return[z]];


OrderBasis::usage = "OrderBasis[basis] returns the ordered list of 64 parameter combinations which compose the basis <basis>.
The possibles values for <basis> are: II, Tsym, Tprod, Tmpo.

EX.1
OrderBasis[II]
OrderBasis[Tprod]
OrderBasis[Tsym]
OrderBasis[Tmpo]
";
OrderBasis[basis_:II]:=Block[{name,Order},
name=SymbolName[basis];
Switch[{name},
{"II"},
Order=Flatten[{Table[Table[{i,Nb2Symb[n]},{n,1,3}],{i,1,3}],Table[Table[{1,Nb2Symb[n1],2,Nb2Symb[n2]},{n2,1,3}],{n1,1,3}],Table[Table[{1,Nb2Symb[n1],3,Nb2Symb[n2]},{n2,1,3}],{n1,1,3}],Table[Table[{2,Nb2Symb[n1],3,Nb2Symb[n2]},{n2,1,3}],{n1,1,3}],
Flatten[Table[Table[Table[{Nb2Symb[n1],Nb2Symb[n2],Nb2Symb[n3]},{n3,1,3}],{n2,1,3}],{n1,1,3}],1],{{{E}}}},2],
{"IInorm"},
Order=Flatten[{Table[Table[{i,Nb2Symb[n]},{n,1,3}],{i,1,3}],Table[Table[{1,Nb2Symb[n1],2,Nb2Symb[n2]},{n2,1,3}],{n1,1,3}],Table[Table[{1,Nb2Symb[n1],3,Nb2Symb[n2]},{n2,1,3}],{n1,1,3}],Table[Table[{2,Nb2Symb[n1],3,Nb2Symb[n2]},{n2,1,3}],{n1,1,3}],
Flatten[Table[Table[Table[{Nb2Symb[n1],Nb2Symb[n2],Nb2Symb[n3]},{n3,1,3}],{n2,1,3}],{n1,1,3}],1],{{{E}}}},2],
{"Tprod"},
Order=Flatten[
{Flatten[Table[Table[{s,1,M},{M,-1,1}],{s,1,3}],1],(*1 to 9*)
Flatten[Table[Table[{1,2,L,M},{M,-L,L}],{L,0,2}],1],(*10 to 18*)
Flatten[Table[Table[{1,3,L,M},{M,-L,L}],{L,0,2}],1],(*19 to 27*)
Flatten[Table[Table[{2,3,L,M},{M,-L,L}],{L,0,2}],1],(*28 to 36*)
Flatten[Table[{1,2,3,J1,M,{J,1}},{J,0,2},{J1,Abs[J-1],J+1},{M,-J1,J1}],2],
{{0,0,0}}},1],
{"Tsym"},
Order=Flatten[{Flatten[Table[Table[{s,1,M},{M,-1,1}],{s,1,3}],1],Flatten[Table[Table[{1,2,L,M},{M,-L,L}],{L,0,2}],1],Flatten[Table[Table[{1,3,L,M},{M,-L,L}],{L,0,2}],1],Flatten[Table[Table[{2,3,L,M},{M,-L,L}],{L,0,2}],1],
Table[{1,2,3,1,M,1},{M,-1,1}],
Table[{1,2,3,3,M,1},{M,-3,3}],
Flatten[Table[Table[{1,2,3,J,M,2},{M,-J,J}],{J,1,2}],1],
Flatten[Table[Table[{1,2,3,J,M,3},{M,-J,J}],{J,1,2}],1],
{{1,2,3,0,0,4}},{{0,0,0}}},1],
{"Tmpo"},
Order=
Flatten[{Flatten[Table[Table[{J,M,S1,S1},{M,-J,J}],{J,0,1}],1],(*1 to 4*)
Flatten[Table[Table[{J,M,S1,S2},{M,-J,J}],{J,0,1}],1],(*5 to 8*)
Flatten[Table[Table[{J,M,S1,S3},{M,-J,J}],{J,1,2}],1],(*9 to 16*)
Flatten[Table[Table[{J,M,S2,S1},{M,-J,J}],{J,0,1}],1],(*17 to 20*)
Flatten[Table[Table[{J,M,S2,S2},{M,-J,J}],{J,0,1}],1],(*21 to 24*)
Flatten[Table[Table[{J,M,S2,S3},{M,-J,J}],{J,1,2}],1],(*25 to 32*)
Flatten[Table[Table[{J,M,S3,S1},{M,-J,J}],{J,1,2}],1],(*33 to 40*)
Flatten[Table[Table[{J,M,S3,S2},{M,-J,J}],{J,1,2}],1],(*41 to 48*)
Flatten[Table[Table[{J,M,S3,S3},{M,-J,J}],{J,0,3}],1]},1](*49 to 64*)
];(*Switch*)
Return[Order];
];


MakeBasis::usage="MakeBasis[basis] returns the ordered list of 64 matrices of the  basis <basis>, in the order defined by OrderBasis[<basis>]. 
* The possible values are II, Tprod, Tsym, Tmpo.

EX.1
MkBasis=MakeBasis[Tprod];
Dimensions[MkBasis]
";
MakeBasis[basis_]:=basis@@@OrderBasis[basis];


MakeBasisi[basis_]:=Block[{i,ibasis },
ibasis = Symbol[ToString[i]<>SymbolName[basis]];
Return[Apply[ibasis,OrderBasis[basis],1]];
];


Basis[II] = MakeBasis[II];
Basisi[II] = MakeBasisi[II];
Basis[IInorm] = MakeBasis[IInorm];
Basisi[IInorm] = MakeBasisi[IInorm];
Basis[Tprod] = MakeBasis[Tprod];
Basisi[Tprod] = MakeBasisi[Tprod];



CoeffList::usage="CoeffList[mtx,basis] returns the list of coefficients of a 8\[Times]8 matrix <mtx> in the basis <basis>.
* The possible values of <basis> are II, Tprod, Tsym, Tmpo.

EX.1
CoeffList[II[1,x],II]
Length[%]
";
CoeffList[mtx_,basis_:Tsym]:=ParallelMap[Chop[FullSimplify[Prod[#,mtx]/Prod[#,#]]]&,Basis[basis]];


Decomposition::usage = "Decomposition[mtx,basis] returns the symbolic decomposition of a matrix in the given basis. 
The output format are defined by the function i<basis> (e.g.  iII,iTprod,iTsym,iTmpo)

EX.1
Decomposition[II[1,x,2,z],II]
Decomposition[II[1,x,2,z],Tprod]
Decomposition[II[1,x,2,z],Tsym]
Decomposition[II[1,x,2,z],Tmpo]
";
Decomposition[mtx_,basis_]:=Module[{coeff,basisi},
basisi=Basisi[basis];
coeff=CoeffList[mtx,basis];
(*Return[FullSimplify[Apply[Plus,coeff*basisi]]];*)
Return[Apply[Plus,coeff*basisi]];
];


IndicePermute::usage = 
"IndicePermute[param,perm] takes parameters <param> defining a Cartesian operator and returns the permuted parameters according to the permutation <perm>. The permutation should be given into its cycle decomposition.

EX.1
IndicePermute[{3,z},Cycles[{{1,3}}]]

EX.2 
IndicePermute[{1,y,2,z},Cycles[{{1,2}}]]

EX.3 
IndicePermute[{x,y,z},Cycles[{{1,2}}]]
";
IndicePermute[indice_,Perm_]:=Block[ {l},l=Length[indice];
Switch[{l},
{2}, 
Return[{PermutationReplace[indice[[1]],Perm],indice[[2]]}];,
{4}, 
Return[{PermutationReplace[indice[[1]],Perm],indice[[2]], PermutationReplace[indice[[3]],Perm],indice[[4]]}];,
{3}, Return[{indice[[PermutationReplace[1,InversePermutation[Perm]]]],indice[[PermutationReplace[2,InversePermutation[Perm]]]],indice[[PermutationReplace[3,InversePermutation[Perm]]]]}];];]


ParticlePermute::usage=
"ParticlePermute[mtx,perm] takes a 8\[Times]8 matrix <mtx> and a permutation <perm> and returns the operator obtained by permuting the spins in the operator.
The permutation <perm> is given by a set of the form {a,b} or {a,b,c}, corresponding to its cyclic decomposition.

EX.1 
Decomposition[II[1,x],II]
Decomposition[ParticlePermute[II[1,x],{1,2}],II]

EX.2
Decomposition[II[1,x,3,z],II]
Decomposition[ParticlePermute[II[1,x,3,z],{1,3}],II]

EX.3 
Decomposition[II[x,y,z],II]
Decomposition[ParticlePermute[II[x,y,z],{1,2,3}],II]
";

ParticlePermute[mtx_,cycle_]:=Block[{permutation,coeff,Newbasis},
permutation = Cycles[{cycle}];
coeff=CoeffList[mtx,II];
Newbasis=Map[IndicePermute[#,permutation]&,OrderBasis[II]];
Newbasis = Parallelize[Apply[II,Newbasis,1]];
Return[Simplify[Parallelize[Apply[Plus,coeff*Newbasis]]]]
];(*Block*)



symmetrize::usage="> symmetrize[mtx,k,l] returns the operator obtained from <mtx> by symmetrizing over the particles <k> and <l>.
> symmetrize[mtx] symmetrizes over the three particles.

EX.1
Decomposition[II[1,x],II]
Decomposition[symmetrize[II[1,x],1,3],II]

EX.2
Decomposition[II[1,x],II]
Decomposition[symmetrize[II[1,x]],II]
";
symmetrize[mtx_,spin1_,spin2_]:=Block[{m,m12,fac},
m = mtx;
m12 = ParticlePermute[mtx,{spin1,spin2}];
fac=1/2;
Return[FullSimplify[fac*(m+m12)]]
]
symmetrize[mtx_]:=Return[Projector1[mtx]];



antisymmetrize::usage="> antisymmetrize[mtx,k,l] returns the operator obtained from <mtx> by anti-symmetrizing over the particles <k> and <l>.
> antisymmetrize[mtx] anti-symmetrizes over the three particles.

EX.1
Decomposition[II[1,x],II]
Decomposition[antisymmetrize[II[1,x],1,3],II]

EX.2
Decomposition[II[x,y,z],II]
Decomposition[antisymmetrize[II[x,y,z]],II]
";
antisymmetrize[mtx_,spin1_,spin2_]:=Block[{m,m12,fac},
m = mtx;
m12 = ParticlePermute[mtx,{spin1,spin2}];
fac = 1/2;
Return[FullSimplify[fac*(m-m12)]]
]
antisymmetrize[mtx_]:=Return[Projector4[mtx]];


Projector::usage="Projector1, Projector2, Projector3 and Projector4 are the four projectors corresponding to the four Young tableaux of dimension 3, respectively to the Tableaux [123],[12][3],[13][2],[1][2][3].
These projectors are used to symmetrize the tensor operators in the Tprod basis to construct the Tsym tensor operators.

EX.1
M = II[1,x];
Decomposition[M,II]
Decomposition[Projector1[M],II]
Decomposition[Projector2[M],II]
Decomposition[Projector3[M],II]
Decomposition[Projector4[M],II]
";


(* [123]*)
Projector1[mtx_]:=Block[{m,m12,m23,m31,m123,m321,fac},
m = mtx;
m12 = ParticlePermute[mtx,{1,2}];
m23 = ParticlePermute[mtx,{2,3}];
m31 = ParticlePermute[mtx,{3,1}];
m123 = ParticlePermute[mtx,{1,2,3}];
m321 = ParticlePermute[mtx,{3,2,1}];
fac = 1/6;
Return[FullSimplify[fac*(m+m12+m23+m31+m123+m321)]]
]
(*[12,3]*)
Projector2[mtx_]:=Block[{m,m12,m31,m123,m321,fac},
m = mtx;
m12 = ParticlePermute[mtx,{1,2}];
m31 = ParticlePermute[mtx,{3,1}];
m123 = ParticlePermute[mtx,{1,2,3}];
m321 = ParticlePermute[mtx,{3,2,1}];
fac = 1/3;
Return[fac*(m+m12-m31-m321)]
];
(*[13,2]*)
Projector3[mtx_]:=Block[{m2,m23,fac},
m2= Projector2[mtx];
m23= ParticlePermute[m2,{2,3}];
fac = Sqrt[4/3];
Return[(1/2*m2+m23)*fac];
];
(*[1,2,3]*)
Projector4[mtx_]:=Block[{m,m12,m23,m31,m123,m321,fac},
m = mtx;
m12 = ParticlePermute[mtx,{1,2}];
m23 = ParticlePermute[mtx,{2,3}];
m31 = ParticlePermute[mtx,{3,1}];
m123 = ParticlePermute[mtx,{1,2,3}];
m321 = ParticlePermute[mtx,{3,2,1}];
fac = 1/6;
Return[fac*(m-m12-m23-m31+m123+m321)]
];


Tsym::usage="Tsym are symmetrized tensors corresponding to the LiSA basis.
Type OrderBasis[Tsym] to see the possible parameters characterizing these operators.
The basis is defined such that the operators of order m=0 are Hermitian.
> Normalized identity: Tsym[1,2,3,0,0,0]
> Linear operators: Tsym[spin,j,m] where <spin>\[Element]{1,2,3}, j=1 and m\[Element]{-1,0,1}.
> Bilinear: Tsym[spin1,spin2,j,m] where (<spin1>,<spin2>)\[Element]{(1,2),(1,3),(2,3)}, j\[Element]{0,1,2} and m\[Element]{-j,...,j}.
> Trilinear: Tsym[1,2,3,j,m,idx] where idx is the symmetry specie of the tensor. 
idx = 1 \[Implies] j\[Element]{1,3}
idx = 2 \[Implies] j\[Element]{1,2}
idx = 3 \[Implies] j\[Element]{1,2}
idx = 4 \[Implies] j\[Element]{0}
and m\[Element]{-j,...,j}.

EX.1
OrderBasis[Tsym]

EX.2
Tsym[3,1,0]
Tsym[1,2,0,0]
Tsym[1,2,3,2,-2,3]
";


(* Normalized and symmetrized, but without prefactor *)
TSYM[spin_,J_,m_]:=normalize[tensor[spin,J,m]];
TSYM[spin1_,spin2_,J_,m_]:=normalize[tensor[spin1,spin2,J,m]];
(* Using Projection operators. Normalized *)
TSYM[spin1_,spin2_,spin3_,J_,m_,idx_]:=Module[{TS},
Switch[{J},
{0},
Switch[{idx},
{0},TS=tensor[spin1,0,0];,
{4},TS=Symbol["Projector"<>ToString[idx]][tensor[spin1,spin2,spin3,J,m,{1,1}]];],
{1},TS=Symbol["Projector"<>ToString[idx]][tensor[spin1,spin2,spin3,J,m,{0,1}]];,
{2},TS=Symbol["Projector"<>ToString[idx]][tensor[spin1,spin2,spin3,J,m,{1,1}]];,
{3},TS=Symbol["Projector"<>ToString[idx]][tensor[spin1,spin2,spin3,J,m,{2,1}]];];
Return[normalize[TS]];
];


pfTsym[spin_,J_,m_]:=Return[1];
pfTsym[spin1_,spin2_,J_,m_]:=Switch[{J},{2},Return[1];,{1},Return[-I];,{0},Return[-1];]; 
pfTsym[spin1_,spin2_,spin3_, J_,m_,idx_]:=Switch[{J,idx},
{0,4},Return[-I];,
{1,1},Return[-1];,
{1,2},Return[1];,
{1,3},Return[1];,
{2,2},Return[-I];,
{2,3},Return[-I];,
{3,1},Return[1];,
{0,0},Return[1];];


Tsym[x__]:= pfTsym[x]*TSYM[x];


Format[iTsym[J_,m_]]:=DisplayForm[SubscriptBox["T",SequenceForm["s,",J,",",m]]];
Format[iTsym[spin_,J_,m_]]:=DisplayForm[SubsuperscriptBox["T",SequenceForm["s,",J,",",m],SequenceForm["{",spin,"}"]]];
Format[iTsym[spin1_,spin2_,J_,m_]]:=DisplayForm[SubsuperscriptBox["T",SequenceForm["s,",J,",",m],SequenceForm["{",spin1,spin2,"}"]]];
Format[iTsym[spin1_,spin2_,spin3_,J_,m_,idx_]]:=DisplayForm[
SequenceForm[
SubsuperscriptBox["T",SequenceForm["s,",J,",",m],SequenceForm["{",spin1,spin2,spin3,"}"]],
"[",idx,"]"]
];
(*EX
iTsym[1,1]
iTsym[1,2,1,1]
iTsym[1,2,3,3,-3,2]
*)


Basis[Tsym] = MakeBasis[Tsym];
Basisi[Tsym] = MakeBasisi[Tsym];


Tmpo::usage="Tmpo is the multipole tensor operator basis for three-spin-1/2 systems.
Type OrderBasis[Tmpo] to see the set of possible parameters corresponding to these tensors.
Tmpo has the form Tmpo[j,m,idx] where idx specifies the transition between the three possible vector sets S1={0,1/2}, S2={1,1/2}, S3={1,3/2}:
 the first value is the parent and the second value is the rank j value of the kets in the set.
Notation: {S1,S2} refers to the transition S1\[Rule]S2
{idx}  ={S1,S1} \[Rule] J=0,1;
        ={S1,S2} \[Rule] J=0,1;
        ={S1,S3} \[Rule] J=1,2;
        ={S2,S1} \[Rule] J=0,1;
        ={S2,S2} \[Rule] J=0,1;
        ={S2,S3} \[Rule] J=1,2;
        ={S3,S1} \[Rule] J=1,2;
        ={S3,S2} \[Rule] J=1,2;
        ={S3,S3} \[Rule] J=0,1,2,3;

EX.1
OrderBasis[Tmpo]

EX.2
Tmpo[2,-2,S3,S3]
";


tmpo3[J_,m_,IDX1_,IDX2_]:=Module[{B,T,i,j,j1,j2,m1,m2,idx,f1,f2},
(*Bi = {j,m,idx} where idx is the father. 
The mother is always 1/2. Here, the father can be 0 ou 1. In fact, they are the possible J values for tmpo2 *)
B= {{3/2,3/2,1},{3/2,1/2,1},{3/2,-1/2,1},{3/2,-3/2,1},{1/2,1/2,1},{1/2,-1/2,1},{1/2,1/2,0},{1/2,-1/2,0}};
idx ={IDX1,IDX2};
Switch[{IDX1},
{S1},idx[[1]]={0,1/2};,
{S2},idx[[1]]={1,1/2};,
{S3},idx[[1]]={1,3/2};
];
Switch[{IDX2},
{S1},idx[[2]]={0,1/2};,
{S2},idx[[2]]={1,1/2};,
{S3},idx[[2]]={1,3/2};
];
T=Table[0,{i,1,Length[B]},{j,1,Length[B]}];
For[i=1,i<= Length[B],i++,
For[j=1,j<= Length[B],j++,
j1=B[[i]][[1]];
m1=B[[i]][[2]];
f1 = B[[i]][[3]];(*f for "father"*)
j2=B[[j]][[1]];
m2=B[[j]][[2]];
f2 = B[[j]][[3]];
If[(m1+m== m2 )&&  (Abs[j1-J]<= j2<= j1+J) &&(idx[[1]]== {f1,j1}) && (idx[[2]]== {f2,j2}),
T[[j,i]]=Sqrt[(2*J +1)/(2*j1 +1)]*ClebschGordan[{j1,m1},{J,m},{j2,m2}]];
];
];
Return[T];
];


(*4*4*)
(* {+++,++-,+-+,+--,-++,-+-,--+,---}*)
tB3p= {{1,0,0,0,0,0,0,0},
{0,Sqrt[1/3],Sqrt[1/3],0,Sqrt[1/3],0,0,0},
{0,0,0,Sqrt[1/3],0,Sqrt[1/3],Sqrt[1/3],0},
{0,0,0,0,0,0,0,1},
{0,2/Sqrt[6],-1/Sqrt[6],0,-1/Sqrt[6],0,0,0},
{0,0,0,1/Sqrt[6],0,1/Sqrt[6],-2/Sqrt[6],0},
{0,0,1/Sqrt[2],0,-1/Sqrt[2],0,0,0},
{0,0,0,1/Sqrt[2],0,-1/Sqrt[2],0,0}};
pB3t=Inverse[tB3p];


TMPO3[x__]:=normalize[pB3t.tmpo3[x].tB3p]; (* 8*8 matrix *)
(*{J} ={0,1} *)


Tmpo[J_,m_,idx1_,idx2_]:=TMPO3[J,m,idx1,idx2];


Format[iTmpo[J_,m_,idx1_,idx2_]]:=DisplayForm[
SequenceForm[
SubscriptBox["T",SequenceForm["m,",J,",",m]],
"[",idx1,",",idx2,"]"]
];

(*EX.
iTmpo[3,3,{{1,3/2},{1,3/2}}]*)


Basis[Tmpo] = MakeBasis[Tmpo];
Basisi[Tmpo] = MakeBasisi[Tmpo];


Hcshift::usage = "Hcshift[spin,nu] returns the chemical shift Hamiltonian for <spin> with Larmor Frequency <nu>.

EX.1
H = Hcshift[3,a]
Decomposition[H,II]
";
Hcshift[spin_,nu_:0]:=2 \[Pi]*nu*II[spin,z];


HcshiftTotal::usage = "HcshiftTotal[nu1,nu2,nu3] returns the sum of the three chemical shift Hamiltonians with Larmor Frequency <nu1> for spin 1, <nu2> for spin 2 and <nu3> for spin 3.

EX.1
H = HcshiftTotal[a,b,c]
Decomposition[H,II]
";
HcshiftTotal[nu1_,nu2_,nu3_]:=Hcshift[1,nu1]+Hcshift[2,nu2]+Hcshift[3,nu3];


Hweak::usage = "Hweak[spin1,spin2,Jc] returns the weak coupling Hamiltonian between <spin1> and <spin2> with coupling constant <Jc> (Hz).

EX.1 
H = Hweak[1,3,a]
Decomposition[H,IInorm]
";
Hweak[spin1_,spin2_,Jc_]:=2\[Pi] *Jc*II[spin1,z,spin2,z];


HweakTotal::usage = "HweakTotal[Jc12,Jc13,Jc23] returns the sum of the three weak coupling Hamiltonians with coupling constants <Jc12> (Hz) between spins 1 and 2, <Jc13> (Hz) between spins 1 and 3 and <Jc23> (Hz) between spins 2 and 3.

EX.1
H = HweakTotal[a,b,c]
Decomposition[H,IInorm]
";
HweakTotal[Jc12_,Jc13_,Jc23_]:=Hweak[1,2,Jc12]+ Hweak[1,3,Jc13]+ Hweak[2,3,Jc23];


Hstrong::usage = "Hstrong[spin1,spin2,Jc] returns the strong coupling Hamiltonian between <spin1> and <spin2> with the coupling constant <Jc> (Hz).

EX.1
Hstrong[2,3,a]
";
Hstrong[spin1_,spin2_,Jc_]:= 2\[Pi] Jc (II[spin1,x,spin2,x]+II[spin1,y,spin2,y]+II[spin1,z,spin2,z]);


HstrongTotal::usage = "HstrongTotal[Jc12,Jc13,Jc23] returns the sum of the strong coupling Hamiltonians with coupling constants <Jc12> (Hz) between spins 1 and 2, <Jc13> (Hz) between spins 1 and 3 and <Jc23> (Hz) between spins 2 and 3.

EX.1 
H = HstrongTotal[a,b,c]
Decomposition[H,IInorm]
";
HstrongTotal[Jc12_,Jc13_,Jc23_]:=Hstrong[1,2,Jc12]+ Hstrong[1,3,Jc13]+ Hstrong[2,3,Jc23];


Hplan::usage = "Hplan[spin1,spin2,Jc] returns the planar coupling Hamiltonian between <spin1> and <spin2> with coupling constant <Jc> (Hz).

EX.1
H = Hplan[2,3,a]
Decomposition[H,IInorm]
";
Hplan[spin1_,spin2_,Jc_]:= 2\[Pi] Jc (II[spin1,x,spin2,x]+II[spin1,y,spin2,y]);


HplanTotal::usage = " HplanTotal[Jc12,Jc13,Jc23] returns the sum of the strong coupling Hamiltonians with coupling constants <Jc12> (Hz) between spins 1 and 2, <Jc13> (Hz) between spins 1 and 3 and <Jc23> (Hz) between spins 2 and 3.

EX.1 
H = HplanTotal[a,b,c]
Decomposition[H,IInorm]
";
HplanTotal[Jc12_,Jc13_,Jc23_]:=Hplan[1,2,Jc12]+ Hplan[1,3,Jc13]+ Hplan[2,3,Jc23];


Hrfxy::usage = "Hrfxy[spin,ux,uy] returns the rf Hamiltonian controlling the spin <spin>, where <ux> (Hz) and <uy> (Hz) is the intensity of the control field in the x- and y-directions. 

EX.1 
H = Hrfxy[3,ux,uy]
Decomposition[H,IInorm]
";
Hrfxy[spin_,ux_,uy_]:=2\[Pi]*(ux II[spin,x]+uy II[spin,y]);(*CHECK FOR PREFACTOR*)


HrfxyTotal::usage = "HrfxyTotal[u1x,u1y,u2x,u2y,u3x,u3y] returns the rf Hamiltonian controlling the three spins, where <ukx> (Hz) and <uky> are the amplitude of the control field acting on spin k in the x- and y-direction respectively.

EX.1 
H =HrfxyTotal[u1x,u1y,u2x,u2y,u3x,u3y]
Decomposition[H,IInorm]
";
HrfxyTotal[u1x_,u1y_,u2x_,u2y_,u3x_,u3y_]:= Hrfxy[1,u1x,u1y]+Hrfxy[2,u2x,u2y]+Hrfxy[3,u3x,u3y];


Hrf::usage = "Hrf[spin,ampl,phase] returns the rf Hamiltonian controlling the spin <spin>, where <ampl> (Hz) is the amplitude of the control field and <phase> (Deg) the phase of the control field.

EX.1 
H = Hrf[2,B,\[Theta]]
Decomposition[H,IInorm]
";
Hrf[spin_,ampl_,phase_]:=Module[{},
Return[Hrfxy[spin,ampl*Cos[phase*Pi/180],ampl*Sin[phase*Pi/180]]];
];


HrfTotal::usage = "Hrf[ampl1,phase1,ampl2,phase2,ampl3,phase3] returns the rf Hamiltonian controlling the three spins,
where <amplk> (Hz) is the amplitude of the control field acting on spin k and <phasek> (Deg) its phase.

EX.1 
H = HrfTotal[B1,\[Theta]1,B2,\[Theta]2,B3,\[Theta]3]
Decomposition[H,IInorm]
";
HrfTotal[ampl1_,phase1_,ampl2_,phase2_,ampl3_,phase3_]:= Hrf[1,ampl1,phase1]+Hrf[2,ampl2,phase2]+Hrf[3,ampl3,phase3];


Evolve::usage = "Evolve[mtx,H,t] returns the matrix resulting of the evolution of <mtx> under the constant Hamiltonian <H> for a time <t>.

EX.1
rho=II[2,x];
nu = 100;
H=Hcshift[2,nu];
t=1/(8*nu);
rho2=Evolve[rho,H,t];
Decomposition[rho,II]
Decomposition[rho2,II]
";
Evolve[mtx_,H_,t_]:=Module[{Propagator},Propagator=ComplexExpand[MatrixExp[-N[I*H*t]]];
      Return[Propagator.mtx.Dagger[Propagator]];];


MultiEvolve::usage = "MultiEvolve[mtx,{H1,...Hk},{t1,...,tk}] returns the matrix resulting of the successive evolution of <mtx> under the Hamiltonian <H1> for a time <t1>, then the Hamiltonian <H2> for a time <t2>, etc.

EX.1
rho=II[1,z]+II[2,z];
ampl = 100;
H1=HrfTotal[ampl,90,ampl,90,0,0];
t1=1/(4*ampl);
Jc = 50;
Hc = Hplan[1,2,Jc];
t2 = 1/(2*Jc);
rho2=MultiEvolve[rho,{H1,Hc},{t1,t2}];
Decomposition[rho,II]
Decomposition[rho2,II]
";
MultiEvolve[mtx_,H_,t_]:=Module[{len,mtxprime},
len=Length[t];
mtxprime=mtx;
For[k=1,k<=len,k++,mtxprime=Evolve[mtxprime,H[[k]],t[[k]]];];
Return[mtxprime];
]


Acquire::usage = "Acquire[mtx,H,target,t,npt] returns the list of <npt>+1 coefficients for <target> in the <mtx> decomposition,  when <mtx> evolves under the Hamiltonian <H> for a time <t>.

Default values: 
	H       = IdentityMatrix[8]
	target  = IdentityMatrix[8]
	t       = 0
	npt     = 0

EX.1
mtx = II[1,x];
nu=10;
H = Hcshift[1,nu];
target = II[1,y];
t = 1/(2*nu);
npt = 8;
Acquire[mtx,H,target,t,npt]
";
Acquire[mtx_,H_: IdentityMatrix[8],target_: IdentityMatrix[8],t_: 0,npt_: 0]:=Module[{dt,dummy1,dummy2,m1,m2,coefflist},
If[npt!=0,dt=t/npt;,dt=0];
dummy1=ComplexExpand[MatrixExp[-N[I*H*dt]]];
dummy2=Dagger[dummy1];
m1=ParallelTable[Chop[Prod[target,mtx]/Prod[target,target]],{i,1,1}];
m2=ParallelTable[Chop[Prod[target,(dummy1^(i)).mtx.(dummy2^(i))]/Prod[target,target]],{i,1,npt}];
coefflist=Flatten[ArrayFlatten[{{m1,m2}}]];
Return[coefflist];]


AcquireAll::usage = "AcquireAll[mtx,H,t,npt,basis] returns a 64\[Times](<npt>+1) matrix. Each column corresponds to the coefficients of <mtx> in the basis <basis> after evolution for time dt=<t>/<npt>.
The possible values for basis are II,Tprod,Tsym,Tmpo.
Type OrderBasis[<basis>] to see the order in which the basis elements appear.

Default values: 
	H      = IdentityMatrix[8]
	t       = 0
	npt   = 0
	basis = Tsym

EX.1
mtx = II[1,x];
Jc = 20;
H = Hweak[1,2,Jc];  
t = 1/(2*Jc);
npt = 4;
A = AcquireAll[mtx,H,t,npt,IInorm];
A[[1]] (*coeff. of II[1,x]*)
A[[2]] (*coeff. of II[1,y]*)
A[[3]] (*coeff. of II[1,z]*)
A[[15]] (*coeff. of 2 II[1,y,2,z]*)
";

AcquireAll[mtx_,H_:IdentityMatrix[8],t_:0,npt_:0,basis_:Tsym]:=Module[{dt,coefftable,dummy1,dummy2,m1,m2,mtxprime},
If[npt!=0,dt=t/npt;,dt=0];
dummy1=ComplexExpand[MatrixExp[-N[I*H*dt]]];
dummy2=Dagger[dummy1];
mtxprime=mtx;
m1=ParallelTable[Chop[Prod[Basis[basis][[i]],mtxprime]/Prod[Basis[basis][[i]],Basis[basis][[i]]]],{i,1,64},{j,1,1}];
For[k=1,k<=npt,k++,
mtxprime=Evolve[mtxprime,H,dt];
m2=ParallelTable[Chop[Prod[Basis[basis][[i]],mtxprime]/Prod[Basis[basis][[i]],Basis[basis][[i]]]],{i,1,64},{j,1,1}];
m1=ArrayFlatten[{{m1,m2}}];
];(*EndFor*)
Return[N[m1]];
]


MultiAcquireAll::usage = "MultiAcquireAll[mtx,{H1,H2,...,Hk},{t1,t2,...,tk},{npt1,npt2,...,nptk},basis] returns a matrix of dimension 64\[Times](<npt1>+<npt2>+...+<nptk>+1) which corresponds to the evolution of <mtx> under the Hamiltonians {<H1>,<H2>,...,<Hk>} for times {<t1>,<t2>,...,<tk>} with a selection of {<npt1>,...,<nptk>} points for each interval in the given basis. Any element in the npt list can be 0.
The i-th column of the resulting matrix is a list of 64 coefficients of the decomposition of mtx(t) in the basis.
The i-th row corresponds to the list of the (<npt1>+<npt2>+...+<nptk>+1) coefficients of the i-th basis element basis[[i]] as the time goes.
The values for <basis> available with the package are II, IInorm, Tprod, Tsym, Tmpo.

Default values: 
	H     = {IdentityMatrix[8]}
	t     = {0}
	npt   = {0}
	basis = Tsym

EX.1
mtx = II[1,z];
(*From equilibrium to transverse magnetization*)
MAXRF = 10000;
H1 = Hrfxy[1,0,MAXRF];
t1 = 1/4*(1/MAXRF);
npt1 = 4;
(*Evolution under strong coupling*)
Jc = 90;
H2 = Hweak[1,2,Jc];
t2 = 1/(2*Jc);
npt2 = 4;
A = MultiAcquireAll[mtx,{H1,H2},{t1,t2},{npt1,npt2},IInorm];
Dimensions[A]
A[[3]] (*coeff. of II[1,z]*)
A[[1]] (*coeff. of II[1,x]*)
A[[15]] (*coeff. of 2 II[1,y,2,z]*)
";
MultiAcquireAll[mtx_,H_:{IdentityMatrix[8]},t_:{0},npt_:{0},basis_:Tsym]:=Module[
{len,mtxprime,dt,dummy1,dummy2,m1,m2},
len=Length[t];
mtxprime=mtx;
m1=ParallelTable[Chop[Prod[Basis[basis][[i]],mtxprime]/Prod[Basis[basis][[i]],Basis[basis][[i]]]],{i,1,64},{j,1,1}];
For[l=1,l<=len,l++,
If[npt[[l]]==0,dt=t[[l]];,dt=(t[[l]]/npt[[l]]);];
dummy1=ComplexExpand[MatrixExp[-N[I*H[[l]]*dt]]];
dummy2=Dagger[dummy1];
mtxprime=dummy1.mtxprime.dummy2;
For[k=1,k<=npt[[l]],k++,
m2=ParallelTable[Chop[Prod[Basis[basis][[i]],mtxprime]/Prod[Basis[basis][[i]],Basis[basis][[i]]]],{i,1,64},{j,1,1}];
m1=ArrayFlatten[{{m1,m2}}];
If[k<npt[[l]],mtxprime=dummy1.mtxprime.dummy2;];
];(*EndFor*)
];(*EndFor*)
Return[N[m1]];]


PartitionCoeffList::usage = "PartitionCoeffList[listCoeff,basis] partition a list of 64 coefficients into a list {D1,D2,...} of droplet coefficients.
The number of droplets, their order and their definition is unique to the basis <basis>.
A list Dk={a1,a2,a3,...} of droplet coefficients assigns each coefficient to spherical harmonics {Y[00],Y[1,-1],Y[1,0],Y[1,1],Y[2,-2],...Y[2,2],Y[3,-3],etc.} in this order.
The list of coefficients <listCoeff> is generally obtained from the output of Decomposition[mtx,basis].
This method was built to be used by the methods PartitionedCoeffList and PlotFromCoeff.

EX.1
LISTB = CoeffList[a Tsym[1,2,0,0] + b Tsym[1,2,2,-2]-c Tsym[1,2,2,2],Tsym]
PartitionCoeffList[LISTB,Tsym]

EX.2
LIST = CoeffList[II[1,x],Tsym]
PartitionCoeffList[LIST,Tsym]

EX.3
LIST2 = CoeffList[II[1,x],Tmpo]
PartitionCoeffList[LIST2,Tmpo]
 ";
PartitionCoeffList[listCoeff_,basis_]:=Module[{dummy},
Switch[{basis},
{Tprod},
dummy={
Flatten[{{0},Take[listCoeff,{1,3}]}],
Flatten[{{0},Take[listCoeff,{4,6}]}],
Flatten[{{0},Take[listCoeff,{7,9}]}],
Flatten[{Take[listCoeff,{10,18}]}],
Flatten[{Take[listCoeff,{19,27}]}],
Flatten[{Take[listCoeff,{28,36}]}],
Flatten[{{0},Take[listCoeff,{37,39}]}],
Flatten[{Take[listCoeff,{40,48}]}],
Flatten[{{0},Take[listCoeff,{49,63}]}],
Flatten[Take[listCoeff,{64}]]};,
{Tsym},
dummy={Flatten[{{0},Take[listCoeff,{1,3}]}],
Flatten[{{0},Take[listCoeff,{4,6}]}],
Flatten[{{0},Take[listCoeff,{7,9}]}],
Flatten[{Take[listCoeff,{10,18}]}],
Flatten[{Take[listCoeff,{19,27}]}],
Flatten[{Take[listCoeff,{28,36}]}],
Flatten[{{0},Take[listCoeff,{37,39}],Table[0,{5}],Take[listCoeff,{40,46}]}],Flatten[{{0},Take[listCoeff,{47,49}],Take[listCoeff,{50,54}]}],
Flatten[{{0},Take[listCoeff,{55,57}],Take[listCoeff,{58,62}]}],
Flatten[Take[listCoeff,{63}]],Flatten[Take[listCoeff,{64}]]};,
{Tmpo},
dummy={Flatten[{Take[listCoeff,{1,4}]}](*{{0,1/2},{0,1/2}}*),
Flatten[{Take[listCoeff,{5,8}]}](*{{0,1/2},{1,1/2}}*),
Flatten[{{0},Take[listCoeff,{9,16}]}](*{{0,1/2},{1,3/2}}*),
Flatten[{Take[listCoeff,{17,20}]}](*{{1,1/2},{0,1/2}}*),
Flatten[{Take[listCoeff,{21,24}]}](*{{1,1/2},{1,1/2}}*),
Flatten[{{0},Take[listCoeff,{25,32}]}](*{{1,1/2},{1,3/2}}*),
Flatten[{{0},Take[listCoeff,{33,40}]}](*{{1,3/2},{0,1/2}}*),
Flatten[{{0},Take[listCoeff,{41,48}]}](*{{1,3/2},{1,1/2}}*),
Flatten[{Take[listCoeff,{49,64}]}]}(*{{1,3/2},{1,3/2}}*);
];(* End switch *)
Return[dummy];]


PartitionedCoeffList::usage = "PartitionedCoeffList[mtx,basis] returns the coefficients of <mtx> in the basis <basis>, already partitioned into the list {D1,D2,...} of droplet coefficients.
The number of droplets, their order and their definition is unique to the basis <basis>.
A list Dk={a1,a2,a3,...} of droplet coefficients assigns each coefficient to spherical harmonics {Y[00],Y[1,-1],Y[1,0],Y[1,1],Y[2,-2],...Y[2,2],Y[3,-3],etc.} in this order.

EX.1
mtx =  II[1,x];
PartitionedCoeffList[mtx,Tprod]
PartitionedCoeffList[mtx,Tsym]
PartitionedCoeffList[mtx,Tmpo]
 ";
PartitionedCoeffList[mtx_,basis_]:=PartitionCoeffList[CoeffList[mtx,basis],basis];


SphFct::usage = "SphFct[listCoeff,p,q] takes a list of coefficients and returns a linear combination of spherical harmonics according to those coefficients in the variables p and q.
The spherical harmonics are ordered regarding to (J,m) as (0,0), (1,-1),(1,0),(1,1), (2,-2), 2,-1), [...],(2,2), (3,-3), ...

EX.1
SphFct[{0,1},\[Theta],\[Phi]]
SphFct[{0,1,2,-1,0,0,4,0,0,-2,-1,1,-1},\[Theta],\[Phi]]

EX.2
mtx = II[1,x];
DL = PartitionedCoeffList[mtx,Tsym]
SphFct[DL[[1]],\[Theta],\[Phi]]

EX.3
mtx = II[E];
DL = PartitionedCoeffList[mtx,Tsym]
SphFct[DL[[11]],\[Theta],\[Phi]]
";

SphFct[listCoeff_,p_,q_]:=Module[{J,M,m,dummy,dummy1,F},
If[Length[listCoeff]>0,
J=(Ceiling[Sqrt[Length[listCoeff]]])-1;(*largest J*)
M=Length[listCoeff]-J-J^2-1;(*last M of the largest J*)

dummy=Flatten[ {Table[SphericalHarmonicY[j,m,p,q],{j,0,J-1},{m,-j,j}],
{Table[SphericalHarmonicY[J,m,p,q],{m,-J,M}]}},2];
dummy1=Plus@@(listCoeff*dummy);
Return[Simplify[dummy1]];,
(*Return[Evaluate[Simplify[dummy1]]];*)
Return[.1]]];


CartFct::usage = "CartFct[listCoeff,p,q,t] takes a list of coefficient <listCoeff> and converts it into the corresponding linear combination of spherical harmonics (in the variables <p> and <q>). It returns the parametrical form of the resulting function {fx,fy,fz}, translated by the vector <t>={tx,ty,tz}. In order, the coefficients are associated to (j,m)={(0,0),(1,-1),(1,0),(1,1),(2,-2),etc.}

Default values: 
t = {0,0,0}

EX.1
SphFct[{1},\[Theta],\[Phi]]
CartFct[{1},\[Theta],\[Phi]]

EX.2
SphFct[{0,1},\[Theta],\[Phi]]
CartFct[{0,1},\[Theta],\[Phi]]
";
CartFct[listCoeff_,p_,q_,t_:{0,0,0}]:=
If[Length[listCoeff]>0,Return[{
Abs[SphFct[listCoeff,p,q] ]Sin[p] Cos[q]+t[[1]],
Abs[SphFct[listCoeff,p,q]] Sin[p] Sin[q]+t[[2]],
Abs[SphFct[listCoeff,p,q]] Cos[p]+t[[3]]}];,
Return[.1]];


Options[DropletColorQuick]={PlotPoints->10,Mesh->None,PlotRange->All,ColorFunctionScaling->False,coloring->1,PlotStyle->Directive[RGBColor[1,0,0],Opacity[0.9],Specularity[White,10]], ViewPoint->{2,-25,7},Axes->False,AxesOrigin->{0,0,0},AxesLabel->Automatic,BoxRatios->Automatic,Boxed->False,GroundSphere->GrayLevel[0.4,0.4],TextColor->White,TextRadius->{0,-0.72,0.975*15/24},Background->Black};


Options[DropletGrayQuick]={PlotPoints->10,Mesh->None,PlotRange->All,ColorFunctionScaling->False,coloring->0,PlotStyle->Directive[Gray,Opacity[0.9],Specularity[White,0],Glow[],Lighting->"Neutral"], ViewPoint->{2,-25,7}(*{2,-20,5}*),Axes->False,AxesOrigin->{0,0,0},AxesLabel->Automatic,BoxRatios->Automatic,Boxed->False,GroundSphere->GrayLevel[0.4,0.4],TextColor->White,TextRadius->{0,-0.72,0.975*15/24},Background->Black};


Options[DropletColor]={PlotPoints->30,Mesh->None,PlotRange->All,ColorFunctionScaling->False,coloring->1,PlotStyle->Directive[RGBColor[1,0,0],Opacity[0.9],Specularity[White,10]], ViewPoint->{2,-25,7},Axes->False,AxesOrigin->{0,0,0},AxesLabel->Automatic,BoxRatios->Automatic,Boxed->False,GroundSphere->GrayLevel[0.4,0.4],TextColor->White,TextRadius->{0,-0.72,0.975*15/24},Background->Black};


Options[DropletGray]={PlotPoints->30,Mesh->None,PlotRange->All,ColorFunctionScaling->False,coloring->0,PlotStyle->Directive[Gray,Opacity[0.9],Specularity[White,0],Glow[],Lighting->"Neutral"], ViewPoint->{2,-25,7}(*{2,-20,5}*),Axes->False,AxesOrigin->{0,0,0},AxesLabel->Automatic,BoxRatios->Automatic,Boxed->False,GroundSphere->GrayLevel[0.4,0.4],TextColor->White,TextRadius->{0,-0.72,0.975*15/24},Background->Black};


Options[DROPSTsym]={PositionsAndNames->{{2 Cos[7 \[Pi]/6],2 Sin[7 \[Pi]/6],0,"1"},{2 Cos[11 \[Pi]/6],2 Sin[11 \[Pi]/6],0,"2"},{2 Cos[\[Pi]/2],2 Sin[\[Pi]/2],0,"3"},{Cos[9 \[Pi]/6],Sin[9 \[Pi]/6],0,"12"},{Cos[5 \[Pi]/6],Sin[5 \[Pi]/6],0,"13"},{Cos[\[Pi]/6],Sin[\[Pi]/6],0,"23"},{0,-2.0,1.8,"(123)"},{0,-0.65,1.8,"(12)3"},{0,0.65,1.8,"{12}3"},{0,2.0,1.8,"{123}"},{0,0,0,"Id"}},
SpheresToPlot->{1,1,1,0,0,0,1,1,1,1,0},
NamesToPlot->     {1,1,1,0,0,0,1,1,1,1,0},
LabelPositions->{(*1*){0,-0.72,0.61},(*2*){0,-0.72,0.61},(*3*){0,-0.72,0.61},(*12*){0,-0.72,0.61},(*13*){0,-0.72,0.61},(*12*){0,-0.72,0.61},(*[123]*){0,-0.72,0.81},(*[12][3]*){0,-0.72,0.81},(*[13][2]*){0,-0.72,0.81},(*[1][2][3]*){0,-0.72,0.81},(*Id*){0,-0.72,0.61}},
ImageSize->200(*Automatic*),
PlotRange->All,BoxRatios->Automatic,Boxed->False,Axes->False,AxesOrigin->{0,0,0},AxesLabel->Automatic,Ticks->False,ViewPoint->{4,-4,2},Background->Black};


Options[DROPSTprod]={PositionsAndNames->{{2Cos[7 \[Pi]/6],2 Sin[7 \[Pi]/6],0,"1"},
{2 Cos[11 \[Pi]/6],2 Sin[11 \[Pi]/6],0,"2"},
{2 Cos[\[Pi]/2],2 Sin[\[Pi]/2],0,"3"},{Cos[9 \[Pi]/6],Sin[9 \[Pi]/6],0,"12"},{Cos[5 \[Pi]/6],Sin[5 \[Pi]/6],0,"13"},{Cos[\[Pi]/6],Sin[\[Pi]/6],0,"23"},
{0,-1.8,1.8,"\{0,1\}"},
{0,-0.5,1.8,"\{1,1\}"},
{0,0.8,1.8,"\{2,1\}"},
{0,0,0,"Id"}},
SpheresToPlot->{1,1,1,0,0,0,1,1,1,0},
NamesToPlot->{1,1,1,0,0,0,1,1,1,0},LabelPositions->{(*1*){0,-0.72,0.61},(*2*){0,-0.72,0.61},(*3*){0,-0.72,0.61},(*12*){0,-0.72,0.61},(*13*){0,-0.72,0.61},(*23*){0,-0.72,0.61},(*{0,1}*){0,-0.62,0.81},(*{1,1}*){0,-0.62,0.81},(*{2,1}*){0,-0.62,0.81},(*Id*){0,-0.72,0.61}},
ImageSize->200(*Automatic*),
PlotRange->All,BoxRatios->Automatic,Boxed->False,Axes->False,AxesOrigin->{0,0,0},AxesLabel->Automatic,Ticks->False,ViewPoint->{4,-4,2},Background->Black};


Options[DROPSTmpo]={PositionsAndNames->{
(*1*){2.5 Cos[17 \[Pi]/12],2.5 Sin[17 \[Pi]/12],0,"1/2[0]"},
(*2*){1.9 Cos[21 \[Pi]/12],1.9 Sin[21 \[Pi]/12],0.7,"->"}(*1/2[0]->1/2,[1]*),
(*3*){2.2 Cos[13 \[Pi]/12],2.2Sin[13 \[Pi]/12],1.3,"->"}(*1/2[0]->3/2*),
(*4*){1.7 Cos[21 \[Pi]/12],1.7 Sin[21 \[Pi]/12],-0.75,"<-"}(*1/2[1]->1/2[0]*),
(*5*){2.5 Cos[25 \[Pi]/12],2.5 Sin[25 \[Pi]/12],0,"1/2[1]"},
(*6*){2.2 Cos[5 \[Pi]/12],2.2 Sin[5 \[Pi]/12],1.3,"<-"}(*1/2[1]->3/2*),
(*7*){1.1 Cos[13 \[Pi]/12],1.1 Sin[13 \[Pi]/12],0.65,"<-"}(*3/2->1/2[0]*),
(*8*){1.1 Cos[5 \[Pi]/12],1.1 Sin[5 \[Pi]/12],0.65,"->"}(*3/2->1/2[1]*),
(*9*){2 Cos[9 \[Pi]/12],2 Sin[9 \[Pi]/12],2,"3/2"}},
SpheresToPlot->{1,1,1,1,1,1,1,1,1}(*{1,0,0,0,1,0,0,0,1}*),
NamesToPlot->{1,1,1,1,1,1,1,1,1}(*{1,0,0,0,1,0,0,0,1}*),(*LabelPositions->{(*1*){0,-0.72,-0.975*15/24},{0,-0.72,0.975*15/24},{0,-0.72,0.975*15/24},{0,-0.72,0.975*15/24},{0,-0.72,-0.975*15/24},{0,-0.72,0.975*15/24},{0,-0.72,0.975*15/24},{0,-0.72,0.975*15/24},{0,-0.72,2*0.975*15/24}},*)
LabelPositions->{(*1/2[0]*){0,0,-1.2},(*1/2[0]->1/2,[1]*){0,0,-0.6},(*1/2[0]->3/2*){0.4,0.4,0.6},(*1/2[1]->1/2[0]*){0,0,0.6},(*1/2[1]*){0,0,-1.2},(*1/2[1]->3/2*){0,-0.72,0.75},(*3/2->1/2[0]*){0,-0.72,-0.41},(*3/2->1/2[1]*){0,0.72,-0.81},(*3/2*){0,0,1.2}},
ImageSize->200(*Automatic*),
PlotRange->All,BoxRatios->Automatic,Boxed->False,Axes->False,AxesOrigin->{0,0,0},AxesLabel->Automatic,Ticks->False,ViewPoint->{4,-4,2},Background->Black};


Options[DROPSariTsym]={PositionsAndNames->{{2 Cos[7 \[Pi]/6],2 Sin[7 \[Pi]/6],0,"1"},{2 Cos[11 \[Pi]/6],2 Sin[11 \[Pi]/6],0,"2"},{2 Cos[\[Pi]/2],2 Sin[\[Pi]/2],0,"3"},{Cos[9 \[Pi]/6],Sin[9 \[Pi]/6],0,"12"},{Cos[5 \[Pi]/6],Sin[5 \[Pi]/6],0,"13"},{Cos[\[Pi]/6],Sin[\[Pi]/6],0,"23"},{0,-2.0,1.8,"(123)"},{0,-0.65,1.8,"(12)3"},{0,0.65,1.8,"{12}3"},{0,2.0,1.8,"{123}"},{0,0,0,"Id"}},
SpheresToPlot->{1,1,1,0,0,0,1,1,1,1,0},
NamesToPlot->     {1,1,1,0,0,0,1,1,1,1,0},
LabelPositions->{(*1*){0,-0.72,0.61},(*2*){0,-0.72,0.61},(*3*){0,-0.72,0.61},(*12*){0,-0.72,0.61},(*13*){0,-0.72,0.61},(*12*){0,-0.72,0.61},(*[123]*){0,-0.72,0.81},(*[12][3]*){0,-0.72,0.81},(*[13][2]*){0,-0.72,0.81},(*[1][2][3]*){0,-0.72,0.81},(*Id*){0,-0.72,0.61}},Options[Plot]};


Options[LinWhite]={ToPlot->1,Shape->{{2 Cos[7 \[Pi]/6],2 Sin[7 \[Pi]/6],0},{2 Cos[11 \[Pi]/6],2 Sin[11 \[Pi]/6],0},{2 Cos[\[Pi]/2],2 Sin[\[Pi]/2],0},{2 Cos[7 \[Pi]/6],2 Sin[7 \[Pi]/6],0}},Color->White,Line->Thick};


Options[LinBlack]={ToPlot->1,Shape->{{2 Cos[7 \[Pi]/6],2 Sin[7 \[Pi]/6],0},{2 Cos[11 \[Pi]/6],2 Sin[11 \[Pi]/6],0},{2 Cos[\[Pi]/2],2 Sin[\[Pi]/2],0},{2 Cos[7 \[Pi]/6],2 Sin[7 \[Pi]/6],0}},Color->Black,Line->Thin};


Options[LinGray]={ToPlot->1,Shape->{{2 Cos[7 \[Pi]/6],2 Sin[7 \[Pi]/6],0},{2 Cos[11 \[Pi]/6],2 Sin[11 \[Pi]/6],0},{2 Cos[\[Pi]/2],2 Sin[\[Pi]/2],0},{2 Cos[7 \[Pi]/6],2 Sin[7 \[Pi]/6],0}},Color->GrayLevel[0.4,0.4],Line->Thick};


Options[LinGrayTmpo]={ToPlot->0,Shape->{
{
(*1*){2 Cos[9 \[Pi]/12],2 Sin[9 \[Pi]/12],2}+{0,-1,0.7},
(*2*){2.3 Cos[11\[Pi]/12-0.2],2.3 Sin[11 \[Pi]/12-0.2],1}+{0,-1,0.7}},
{
(*1*){1.6 Cos[16 \[Pi]/12],1.6 Sin[16 \[Pi]/12],0.8}+{0,-1,0.7},
(*2*){2.6 Cos[17 \[Pi]/12],2.6 Sin[17 \[Pi]/12],0.1}+{0,-1,0.7}}
},Color->GrayLevel[0.4,0.4],Line->Thick};


Options[LinNone]={ToPlot->0};


Options[ColorQuickTsym]={DropletParam->DropletColorQuick,DROPSParam->DROPSTsym,ConnectShape->LinGray};


Options[GrayQuickTsym]={DropletParam->DropletGrayQuick,DROPSParam->DROPSTsym,ConnectShape->LinGray};


Options[ColorTsym]={DropletParam->DropletColor,DROPSParam->DROPSTsym,ConnectShape->LinGray};


Options[GrayTsym]={DropletParam->DropletGray,DROPSParam->DROPSTsym,ConnectShape->LinGray};


Options[ColorQuickTprod]={DropletParam->DropletColorQuick,DROPSParam->DROPSTprod,ConnectShape->LinGray};


Options[GrayQuickTprod]={DropletParam->DropletGrayQuick,DROPSParam->DROPSTprod,ConnectShape->LinGray};


Options[ColorTprod]={DropletParam->DropletColor,DROPSParam->DROPSTprod,ConnectShape->LinGray};


Options[GrayTprod]={DropletParam->DropletGray,DROPSParam->DROPSTprod,ConnectShape->LinGray};


Options[ColorQuickTmpo]={DropletParam->DropletColorQuick,DROPSParam->DROPSTmpo,ConnectShape->LinGrayTmpo};


Options[GrayQuickTmpo]={DropletParam->DropletGrayQuick,DROPSParam->DROPSTmpo,ConnectShape->LinGrayTmpo};


Options[ColorTmpo]={DropletParam->DropletColor,DROPSParam->DROPSTmpo,ConnectShape->LinGrayTmpo};


Options[GrayTmpo]={DropletParam->DropletGray,DROPSParam->DROPSTmpo,ConnectShape->LinGrayTmpo};


MyPlotCoeff::usage = "MyPlotCoeff[listCoeff,<options>] plots the sum of the spherical harmonics according to the coefficient list <listCoeff>.
The possible <options> are:
position      : precising the position of the center of the droplet
dropletOutput : the display Output for the picture.
label         : the label of the droplet.
PlotLabel     : a boolean value (0 or 1) precising if the label should be displayed or not. 
PlotSphere    : a boolean value (0 or 1) precising if a sphere of radius 1 should be displayed or not.
LabelPosition : precise the position of the label relative to the cented of the droplet.
The values for dropletParam already implemented in this package are DropletColorQuick, DropletColor, DropletGrayQuick and DropletGray.

Default option values: 
	position-> {0,0,0}
	label -> '' ''
	dropletOutput -> DropletColorQuick
	PlotName -> 0 % a Flag
    PlotSphere -> 1 % a Flag
    LabelPosition -> {0,0,15/24}
	
EX.1
Options[MyPlotCoeff]
 
EX.2
MyPlotCoeff[{0,1}]

EX.3
mtx = II[2,x]+II[2,y,3,z];
Dec = PartitionedCoeffList[mtx,Tsym];
(*Plot the linear droplet in spin 2*)
MyPlotCoeff[Dec[[2]],Plot]
(*Plot the bilinear droplet in spins (2,3)*)
MyPlotCoeff[Dec[[6]],PlotSphere-> 0]
";
Options[MyPlotCoeff]={position-> {0,0,0},dropletOutput-> DropletColorQuick,label-> "",PlotLabel-> 0,PlotSphere-> 1,LabelPosition-> {0,0,15/24}};

MyPlotCoeff[listCoeff_,OptionsPattern[]]:=Module[{CenterPos,dropletParam,LabelPos,OPTIONS,textPosition,colorfct,a,b,l,is0,P,S,T,PlottingSet},
CenterPos = OptionValue[position];
LabelPos =  OptionValue[LabelPosition];

textPosition=LabelPos+CenterPos;
l=Sqrt[Plus@@Abs[CenterPos*CenterPos]];
(* Color function *)
(* Gray parameters: a>=0 Darker , b<=1 Paler *)
a=1;
b=0.5;
dropletParam = OptionValue[dropletOutput];

If[OptionValue[dropletParam,coloring]==1,
colorfct=Function[{x,y,z,\[Theta],\[Phi],r},Hue[Mod[(Arg[SphFct[listCoeff,\[Theta],\[Phi]]])/(2 Pi),1]]],
(*colorfct=Function[{x,y,z,\[Theta],\[Phi],r},GrayLevel[0.8+Mod[(Arg[SphFct[listCoeff,\[Theta],\[Phi]]])/(2 Pi),1]]]*)
colorfct=Function[{x,y,z,\[Theta],\[Phi],r},
GrayLevel[ a+(2*(b-a))*Mod[(Arg[SphFct[listCoeff,\[Theta],\[Phi]]])/(2 Pi),1] ]]
];
(* Matrix components *)
is0=Plus@@Abs[listCoeff];
If[is0!=  0,
P=ParametricPlot3D[CartFct[listCoeff,\[Theta],\[Phi],CenterPos],{\[Theta],0,\[Pi]},{\[Phi],0,2 \[Pi]},
PlotPoints->OptionValue[dropletParam,PlotPoints],
Mesh->OptionValue[dropletParam,Mesh],
PlotRange->OptionValue[dropletParam,PlotRange],
ColorFunctionScaling->OptionValue[dropletParam,ColorFunctionScaling],
ColorFunction->colorfct,
PlotStyle->OptionValue[dropletParam,PlotStyle]];,
P=ParametricPlot3D[{0,0,0},{\[Theta],0,\[Pi]},{\[Phi],0,2 \[Pi]},
PlotPoints->OptionValue[dropletParam,PlotPoints],
Mesh->OptionValue[dropletParam,Mesh],
PlotRange->OptionValue[dropletParam,PlotRange],
ColorFunctionScaling->OptionValue[dropletParam,ColorFunctionScaling],
ColorFunction->colorfct,
PlotStyle->OptionValue[dropletParam,PlotStyle]];
];
PlottingSet={P};
If[OptionValue[PlotSphere]==1,
S=Graphics3D[{OptionValue[dropletParam,GroundSphere],Sphere[CenterPos,(*radius*)0.5]}];
AppendTo[PlottingSet,S];
];
If[OptionValue[PlotLabel]==1,
T=Graphics3D[{Text[Style[OptionValue[label],FontFamily-> "Helvetica",(*Bold,*)OptionValue[dropletParam,TextColor]],textPosition]}];
AppendTo[PlottingSet,T];
];
Return[Show[PlottingSet,
ViewPoint->OptionValue[dropletParam,ViewPoint],
Axes->OptionValue[dropletParam,Axes],
AxesOrigin->OptionValue[dropletParam,AxesOrigin],
AxesLabel->OptionValue[dropletParam,AxesLabel],
BoxRatios->OptionValue[dropletParam,BoxRatios],
Boxed->OptionValue[dropletParam,Boxed],
Background->OptionValue[dropletParam,Background]
]];
];


PlotFromCoeff::usage="PlotFromCoeff[CoeffList,basis] plots the DROPS representation on an operator from the list of its 64 coefficients obtained by decomposition in the basis <basis>. 
* Default Options: Output    -> ColorQuick
                  Linear    ->1 
                  Bilinear  ->1 
                  Trilinear ->1
                  Id        ->1
                  Labels    ->1
The Output already implemented with this package are: ColorQuick, GrayQuick, Color, Gray.

EX.1
mtx = 4 II[x,y,z]
Dec = CoeffList[mtx,Tmpo]
PlotFromCoeff[Dec,Tmpo,Output\[Rule] GrayQuick]

EX.2
mtx = Tprod[1,2,3,3,1,{2,1}];
Dec = N[CoeffList[mtx,Tsym]]
PlotFromCoeff[Dec,Tsym]
PlotFromCoeff[Dec,Tsym,Linear\[Rule] 0,Bilinear\[Rule] 0,Trilinear\[Rule]1]
";
(*The difference with PlotMatrix is that we are forced to explicitely give the basis *)
Options[PlotFromCoeff]={Output->ColorQuick,Linear->1,Bilinear->1,Trilinear->1,Id->1,Labels->1};
PlotFromCoeff[CoeffList_,basis_,OptionsPattern[]]:=Module[{list64Coeff,partCoeff,NbComp,OPTIONS,i,dropletParam,DropsParam,connectShape,select,coordinates,coeffLists,P,test,Circ,PlotLin,Arc,Triag,T1,componentsToPlot,V,PlottingSet,namesToPlot,spheresToPlot,labelPositions},

OPTIONS=Symbol[SymbolName[OptionValue[Output]]<>SymbolName[ basis]];(*Ex. OPTIONS = ColorQuickTprod*)
(*list64Coeff = N[CoeffList[mtx,Symbol[OptionValue[basis]]]];*)
partCoeff = Chop[N[PartitionCoeffList[CoeffList,basis]]];
NbComp=Length[partCoeff];

(*componentsToPlot=OptionValue[OPTIONS,ComponentsToPlot];*)
dropletParam=OptionValue[OPTIONS,DropletParam];
DropsParam=OptionValue[OPTIONS,DROPSParam];
connectShape=OptionValue[OPTIONS,ConnectShape];

coordinates=OptionValue[DropsParam,PositionsAndNames];
(*coordinates=Append[#,dropletParam]&/@coordinates;*)
spheresToPlot=OptionValue[DropsParam,SpheresToPlot];

(*V is a list of 0 or 1 saying if the component has to be plotted *)
V=Table[1,{i,1,NbComp}]; (*All plotted at the beginning *)
Do[V[[i]]=OptionValue[Linear],{i,1,3}];
Do[V[[i]]=OptionValue[Bilinear],{i,4,6}];
Do[V[[i]]=OptionValue[Trilinear],{i,7,NbComp-1}];
V[[NbComp]]=OptionValue[Id];(*0 or 1*)

(* Labels *)
namesToPlot=OptionValue[Labels]*OptionValue[DropsParam,NamesToPlot];(* can be turned to 0 *)
labelPositions=OptionValue[DropsParam,LabelPositions];

(*Adapt the coefficients list to the components to plot*)
coeffLists=Times[partCoeff,V];

P= Table[0,{Length[coeffLists]}];
For[k=1,k<= Length[coeffLists],k++,
P[[k]]=MyPlotCoeff[coeffLists[[k]],position-> Take[coordinates[[k]],{1,3}],
label-> Take[coordinates[[k]],{4}][[1]],
dropletOutput-> dropletParam,
PlotLabel-> namesToPlot[[k]]*OptionValue[Labels],
PlotSphere-> spheresToPlot[[k]],
LabelPosition-> labelPositions[[k]]
];
];

(*Construction of the plotting set*)
PlottingSet={};(*Initialization*)
If[OptionValue[Linear]==1,
Do[AppendTo[PlottingSet,P[[i]]],{i,1,3}];
];
If[OptionValue[Bilinear]==1,
Do[AppendTo[PlottingSet,P[[i]]],{i,4,6}];
];
If[OptionValue[Trilinear]==1,
Do[AppendTo[PlottingSet,P[[i]]],{i,7,NbComp-1}];
];
If[OptionValue[Id]==1,
AppendTo[PlottingSet,P[[NbComp]]];
];

If[OptionValue[connectShape,ToPlot]*OptionValue[Linear]==1,
Triag=Graphics3D[{OptionValue[connectShape,Color],OptionValue[connectShape,Line],Line[OptionValue[connectShape,Shape]]}];AppendTo[PlottingSet,Triag];
,None];

(*Return[GraphicsGrid[{PlottingSet}]]*)
Return[Show[PlottingSet,PlotRange->OptionValue[DropsParam,PlotRange],PreserveImageOptions->True,ViewPoint->OptionValue[DropsParam,ViewPoint],Boxed->OptionValue[DropsParam,Boxed],BoxRatios->OptionValue[DropsParam,BoxRatios],Axes->OptionValue[DropsParam,Axes],AxesOrigin->OptionValue[DropsParam,AxesOrigin],AxesLabel->OptionValue[DropsParam,AxesLabel],Ticks->OptionValue[DropsParam,Ticks],ViewAngle->All,ImageSize->OptionValue[DropsParam,ImageSize],ImagePadding-> 8,Background->OptionValue[DropsParam,Background]]];
];


PlotMatrix::usage="PlotMatrix[mtx,<Options>] plot the DROPS representation of mtx.
Default Options:
Output    -> ColorQuick                  
Linear    ->1 (doesn't make sens for Tmpo)                
Bilinear  ->1 (doesn't make sens for Tmpo)                
Trilinear ->1 (doesn't make sens for Tmpo)                 
Id        ->1 (doesn't make sens for Tmpo)                
Labels    ->1               
basis     -> Tsym 

EX.1
mtx = II[1,x]+II[2,x]+II[1,z,2,z];
PlotMatrix[mtx,Trilinear->0,Id->0,basis->Tsym]
PlotMatrix[mtx,Trilinear->0,Id->0,basis->Tprod]

EX.2
PlotMatrix[II[1,x]+II[2,x]+II[1,z,2,z],basis->Tmpo]

EX.3
mtx = Tprod[1,2,3,1,1,{1,1}];
PlotMatrix[mtx,Output\[Rule]ColorQuick,basis\[Rule] Tprod]
PlotMatrix[mtx,Output\[Rule]ColorQuick,basis\[Rule] Tsym]
PlotMatrix[mtx,Output\[Rule]ColorQuick,basis\[Rule] Tmpo]
PlotMatrix[mtx,Output\[Rule]ColorQuick,basis\[Rule] Tprod,Linear\[Rule] 0,Bilinear\[Rule] 0]
PlotMatrix[mtx,Output\[Rule]ColorQuick,basis\[Rule] Tprod,Trilinear\[Rule] 0]
";


Options[PlotMatrix]=Join[{basis->Tsym},Options[PlotFromCoeff]];
(*Output can be, ColorQuick,GrayQuick,Color,Gray*)
(*not forced to precise the basis*)
PlotMatrix[mtx_,opts:OptionsPattern[]]:=PlotFromCoeff[Chop[N[CoeffList[mtx,OptionValue[basis]]]],OptionValue[basis],FilterRules[{opts},Options[PlotFromCoeff]]];


ImportPulseSequence::usage="ImportPulseSequence[filename] reads pulse files containing three columns with duration, percentage of amplitude and phase of each pulse slice and return {L,MAX_RF} where L is a n\[Times]3 matrix the lines of which are: {duration(i),amplitude(i),phase(i)}, 1<=i<=n.
***IMPORTANT*** DO NOT forget to specify the maximal (100%) radio frequency [Hz] as a single line somewhere in the pulse file (e.g. #MAX_RF 15000). 
Lines beginning with # are ignored unless a keyword is recognized.
If the file is not in the current directory (see Directory[]), then filename should have the form of the full path to find the directory.

EX.1 
File1 = FileNameJoin[{NotebookDirectory[]<>''/BasicINEPT/''<>''pulse1.txt''}]
ps1 = ImportPulseSequence[File1]

File2 = FileNameJoin[{NotebookDirectory[]<>''/BasicINEPT/''<>''pulse2.txt''}]
ps2 = ImportPulseSequence[File2]
";
ImportPulseSequence[filename_]:=Module[{inputfile,line,lines,items,len,result,time,
keymaxrf="#MAX_RF",maxrf,
keyabstime="#ABS_TIME",absolutetime=False},
AbortProtect[
inputfile=OpenRead[filename];
lines=ReadList[filename,Record,RecordSeparators->"\n"];
len=Length[lines];
result={};
For[i=1,i<=len,i++,
line=StringSplit[ToString[lines[[i]]]];
If[Length[line]!=0(*ignore empty*),
If[Characters[line[[1]]][[1]]=="#",(*check for commands, ignore if no commands found*)
If[line[[1]]==keymaxrf,maxrf=ToExpression[line[[2]]]];
If[line[[1]]==keyabstime,absolutetime=True],
(*else convert line to expression and append*)
AppendTo[result,ToExpression[line]]];
];
];(*For*)
Close[inputfile];
];
If[!absolutetime,
len=Length[result];
time=result[[1,1]];
For[i=2,i<=len,i++,
time+=result[[i,1]];
result[[i,1]]=time;];
];
Return[{result,maxrf}]
](*Module*);



CombinePulseSequences::usage=
"CombinePulseSequences[{pulse1,max_rf1}, {pulse2,max_rf2},{pulse3,max_rf3}, Hoffset, scaling_factor]
takes the list of three pulse sequences <{pulsek,max_rfk}>, a constant internal Hamiltonian <Hoffset> and a scaling factor and pdivides the experiment into parts on which the total Hamiltonian is constant.
The output is a list L such that the i-th element of L has the form: L[[i]]={duration,H,{ampl1,phase1},{ampl2,phase2},{ampl3,phase3}}.
The pulses <pulse1>, <pulse2> and <pulse3> are n\[Times]3 matrices the lines of which are of the form: {duration(i),amplitude(i),phase(i)}, 1<=i<=n.
The scaling factor <scaling_factor> converts the pulses of intensity I and of duration T into pulses of intensity I/<scaling_factor> of duration T\[Times]<scaling_factor>.

This routine combines the following things:
1. Gathers pulse sequences as provided by the ImportPulseSequence function
2. Creates new (scaled) time coordinate to slow down the spin movement while applying radio frequency
3. Creates Hamiltonian from radio frequencies and constant Hoffset term

EX.1
File1 = FileNameJoin[{NotebookDirectory[]<>''/BasicINEPT/''<>''pulse1.txt''}];
File2 = FileNameJoin[{NotebookDirectory[]<>''/BasicINEPT/''<>''pulse2.txt''}];
ps1 = ImportPulseSequence[File1];
ps2 = ImportPulseSequence[File2];
ps3 = {{},0};
Hoffset = HweakTotal[140,0,0]+HcshiftTotal[30,10,0];
scalingfactor = 1;
cps = CombinePulseSequences[ps1,ps2,ps3,Hoffset,scalingfactor]
";

CombinePulseSequences[pulse1_,pulse2_,pulse3_,Hoffset_,scalingfactor_:1]:=Module[{NbSpins,s,Hcur,Hlast,Result,count,len,LEN,t1,t2,t3,n,t0,tlist,dt,step,p,maxrf,nu,phase,nulast,phaselast,threshold},

(*Initialization*)
NbSpins=3;
s=Floor[scalingfactor];
Hlast=DiagonalMatrix[Table[0,{8}]];
Result={};
count=0;
t0=0;
dt=0;

n=Table[1,{k,1,NbSpins}];
(*p = Table[Symbol["pulse"<>ToString[k]][[1]],{k,1,NbSpins}];*)
p = {pulse1[[1]],pulse2[[1]],pulse3[[1]]};
(*maxrf = Table[Symbol["pulse"<>k][[2]],{k,1,NbSpins}];*)
maxrf = {pulse1[[2]],pulse2[[2]],pulse3[[2]]};

threshold=4*\[Pi]^2;(*do not scale rf smaller than 1 Hz; single spin: tr{Hrf\.b2}=4\[Pi]\.b2nu\.b2 for normalized J*)
(*Lengths of the pulse sequences*)
len = Table[Length[p[[k]]],{k,1,NbSpins}];
(*Get all duration points where radio frequency changes in any pulse*)
tlist=Sort[Join[p[[1]][[All,1]],p[[2]][[All,1]],p[[3]][[All,1]]]];
LEN=Length[tlist];

(*Initialization of the dynamical variables*)
nu = Table[0,{i,1,LEN}];
nulast = Table[0,{i,1,LEN}];
phase = Table[0,{i,1,LEN}];
phaselast = Table[0,{i,1,LEN}];

(*For loop over timesteps where the total radio frequecy is constant*)
For[i=1,i<=LEN,i++,
step=tlist[[i]]-t0;
If[step==0,Continue[]];
(*get indices of pulse sequences, check if out of range*)
Do[If[n[[k]]<=len[[k]] && tlist[[i]]>p[[k]][[n[[k]],1]],n[[k]]++;];,{k,1,NbSpins}];
(*get rf components and Hamiltonian*)
Do[
If[n[[k]]<=len[[k]],
nu[[k]]=p[[k]][[n[[k]],2]];
phase[[k]]=p[[k]][[n[[k]],3]];,
nu[[k]]=0;phase[[k]]=0;
];,{k,1,NbSpins}];

(*Current Hamiltonian*)
Hcur=HrfTotal[nu[[1]]*maxrf[[1]]/100,phase[[1]],nu[[2]]*maxrf[[2]]/100,phase[[2]],nu[[3]]*maxrf[[3]]/100,phase[[3]]];

(*if same Hamiltonian as before then proceed to new index, else save in list (except initial hamiltonian)*)
If[Hcur==Hlast,dt+=step,
If[Abs[Prod[Hlast,Hlast]]<threshold,
(*no scaling*)
AppendTo[Result,{dt,Hlast+Hoffset,{nulast[[1]],phaselast[[1]]},{nulast[[2]],phaselast[[2]]},{nulast[[3]],phaselast[[3]]}}],
(*scaling Hamiltonian, but not the pulses to plot original pulse size*)
AppendTo[Result,{s*dt,(Hlast+Hoffset)/s,{nulast[[1]],phaselast[[1]]},{nulast[[2]],phaselast[[2]]},{nulast[[3]],phaselast[[3]]}}]
];
dt=step;
Hlast=Hcur;
Do[nulast[[k]]=nu[[k]];
phaselast[[k]]=phase[[k]];,{k,1,NbSpins}];
];
t0=tlist[[i]];
];(*For*)

(*do not miss last set of Hamiltonians*)
If[Abs[Prod[Hlast,Hlast]]<threshold,
(*no scaling*)
AppendTo[Result,{dt,Hlast+Hoffset,{nulast[[1]],phaselast[[1]]},{nulast[[2]],phaselast[[2]]},{nulast[[3]],phaselast[[3]]}}],
(*scaling Hamiltonian, but not the pulses to plot original pulse size*)
AppendTo[Result,{s*dt,(Hlast+Hoffset)/s+Hoffset,{nulast[[1]],phaselast[[1]]},{nulast[[2]],phaselast[[2]]},{nulast[[3]],phaselast[[3]]}}]
];

AppendTo[Result,{\[Infinity],Hoffset,{0,0},{0,0},{0,0}}];(*for infinitely long evolution*)
Return[Take[Result,1-Length[Result]]];
];


CreateAllMatrices::usage="CreateAllMatrices[InitialDensity,CombinedPulseSequences,points,<Options>] uses the <CombinedPulseSequences>,
  i.e. the output of the CombinePulseSequences method in the format {{time,Hamiltonian,{ampl1,phase1},{ampl2,phase2},{ampl3,phase3}}_i}
  to evolve the initial density operator <InitialDensity> in time.
* <points> specifies the sampling points on the time scale (possibly adjusted by the scaling factor!):
  Either a given number of evenly distributed samples over the whole pulse sequence 
  OR 
  a list of time values (\[Micro]s) at which the matrices are calculated each.
The <Options> are:
* <totalTime> : specifies the overall time of the evolution. If not precised, this time is the minimum time to include all the pulses if there is and is set to 1000 if there is just evolution (no pulses).
* <report> : (bool) progress status is printed if report->True. The default value is report->False.

EX.1
Options[CreateAllMatrices]

EX.2
File1 = FileNameJoin[{NotebookDirectory[]<>''/BasicINEPT/''<>''pulse1.txt''}];


File2 = FileNameJoin[{NotebookDirectory[]<>''/BasicINEPT/''<>''pulse2.txt''}];

ps1 = ImportPulseSequence[File1];
ps2 = ImportPulseSequence[File2];
ps3 = {{},0};

InitialDensity=II[1,z];
Hoffset=HweakTotal[212,0,0]+HcshiftTotal[30,10,0];
scalingfactor = 1;
cps = CombinePulseSequences[ps1,ps2,ps3,Hoffset,scalingfactor];
points = 4;
AllMatrices = CreateAllMatrices[InitialDensity,cps,points,report\[Rule] False];
Dimensions[AllMatrices]

Densities       = AllMatrices[[1]];(*a list with the npt+1 density matrices*)
Propagators     = AllMatrices[[2]];(*a list with the npt+1 propagators*)
EffPropagators  = AllMatrices[[3]];(*a list with the npt+1 effective propagators*)
Hamiltonians    = AllMatrices[[4]];(*a list with the npt+1 actual hamiltonians*)
EffHamiltonians = AllMatrices[[5]];(*a list with the npt+1 effective hamiltonians*)

Map[PlotMatrix[#,basis\[Rule] Tsym,Output\[Rule] ColorQuick]&,Densities]
(*Map[PlotMatrix[#,basis\[Rule] Tsym,Output\[Rule] ColorQuick,Trilinear\[Rule] 0]&,Hamiltonians]
Map[PlotMatrix[#,basis\[Rule] Tsym,Output\[Rule] ColorQuick]&,EffPropagators]*)
";

Options[CreateAllMatrices]={report-> False,totalTime-> Automatic};
CreateAllMatrices[InitialDensity_,CombinedPulseSequences_,timepoints_,OptionsPattern[]]:=Module[
{cps,lencps,ncps,tcps,t0,t1,evo,tot,dt,pts,len,rho,prop,ham,effprop,effham,lrho,lprop,leffprop,leffham,lham,lp1,lp2,lp3,startingtime},

(*Initialization*)
cps=CombinedPulseSequences;
ncps=1;
len=Length[timepoints];
rho=InitialDensity;
effprop=IdentityMatrix[8];
lrho={N[InitialDensity]};
lprop={DiagonalMatrix[Table[0,{8}]]};
leffprop={DiagonalMatrix[Table[0,{8}]]};
leffham={DiagonalMatrix[Table[0,{8}]]};
lham={DiagonalMatrix[Table[0,{8}]]};
lp1={{0,0}};
lp2={{0,0}};
lp3={{0,0}};
startingtime=AbsoluteTime[];

lencps=Length[CombinedPulseSequences]-1;

If[OptionValue[totalTime]==Automatic,
If[Total[cps[[1;;lencps,1]]]>0,
tot=Total[cps[[1;;lencps,1]]];,
tot=1000;];,
tot=OptionValue[totalTime];
];

If[len>0,pts=timepoints,
dt=tot/timepoints;
pts=Table[i*dt,{i,timepoints}];
len=timepoints;
];

tcps=cps[[ncps,1]];
t1=0;(*end of last evolution*)
For[i=1,i<=len,i++,
t0=t1;(*beginning of new evolution*)
t1=pts[[i]];

                                                                   (*Hamiltonian changes during the timestep?*)
If[t1<=tcps,
                                                                (*no*)
evo={{t1-t0,cps[[ncps,2]]}},
                                                                (*yes*)
evo={};
(*check for complete steps*)
While[t1>tcps,
AppendTo[evo,{tcps-t0,cps[[ncps,2]]}];
t0=tcps;
ncps++;
tcps+=cps[[ncps,1]];
];
AppendTo[evo,{t1-t0,cps[[ncps,2]]}];
];

(*Calculate
 the Effective Propagator of the timeslice *)
prop=IdentityMatrix[8];
For[j=Length[evo],j>=1,j--,
prop=Dot[prop,ComplexExpand[MatrixExp[-N[I*evo[[j,2]]*evo[[j,1]]/1000000]]]];
];

(* the Effective Hamiltonian of the timeslice *)
ham=MatLog[prop]*1000000I/Total[evo[[All,1]]];
ham-=Tr[ham]*IdentityMatrix[8]/8;(*find traceless Hamiltonian*)

(* the evolved Density Operator*)
rho=Dot[prop,rho,Dagger[prop]];

(* the Effective over-all Propagator and Hamiltonian *)
effprop=Dot[prop,effprop];
effham=MatLog[effprop]*1000000I/t1;
effham-=Tr[effham]*IdentityMatrix[8]/8;(*find traceless Hamiltonian*)

(*Append
 to lists for subroutines*)
AppendTo[lrho,N[rho]]; 
AppendTo[lprop,N[prop]]; 
AppendTo[lham,N[ham]]; 
AppendTo[leffprop,N[effprop]]; 
AppendTo[leffham,N[effham]]; 
];(*For*)

If[OptionValue[report],
Print["Listing and evolution done ("<>ToString[AbsoluteTime[]-startingtime]<>" sec)"];
startingtime=AbsoluteTime[];
];

Return[{lrho,lprop,leffprop,lham,leffham}];
];


SlidesEvolve::usage = "SlidesEvolve[mtx,H,t,npt,slideName,<Options>] returns a list of <npt>+1 pictures (.png by default) in order to create animations. The slides are stored in the current directory (type Directory[] to see it and SetDirectory[...] to change it).

Default values:
H = Sqrt[2] (II[1,z]+II[2,z]+II[3,z])
t = 2 \[Pi]
npt = 16
slideName = ''slides''

The <Options> are 
basis : the basis in which to decompose the matrix <mtx>
Extension : Set to ''.png'' by default.
Resolution : Plotting resolution, set to 200 by default.
And all the options of the PlotFromCoeff method.

EX.1
Options[SlidesEvolve]

EX.2
SlidesEvolve[4*II[x,y,x],Sqrt[2] (II[1,z]+II[2,z]+II[3,z]),2 \[Pi],0,''slidesA'',basis\[Rule] Tsym,Resolution\[Rule] 200]
EX.3
SlidesEvolve[II[x,y,x],Sqrt[2] (II[1,z]+II[2,z]+II[3,z]),2 \[Pi],0,''slidesB'',basis\[Rule] Tsym,Resolution\[Rule] 200]
EX.4
SlidesEvolve[II[x,y,x],Sqrt[2] (II[1,z]+II[2,z]+II[3,z]),2 \[Pi],0,''slidesC'',Output\[Rule] GrayQuick,Extension\[Rule] ''.pdf'']
";

Options[SlidesEvolve]=Join[{basis-> Tsym,Extension-> ".png",Resolution-> 200},Options[PlotFromCoeff]];
SlidesEvolve[mtx_,H_:Sqrt[2] (II[1,z]+II[2,z]+II[3,z]),t_:2 \[Pi],npt_:16,slidesName_:"slides",opts:OptionsPattern[]]:=Module[{nbDigit,X,L,str},
(*Number of digits to use *)
nbDigit=Length[IntegerDigits[npt+1]];
X=Transpose[AcquireAll[mtx,H,t,npt,OptionValue[basis]]];
For[i=1,i<=npt+1,i++,
str=ToStr[i,nbDigit,"0"];
L=PlotFromCoeff[X[[i]],OptionValue[basis],FilterRules[{opts},Options[PlotFromCoeff]]];
Export[slidesName <>"_"<>str<>OptionValue[Extension],L,ImageResolution->OptionValue[Resolution]];
];(*For*)
];


SlidesCoeff::usage = "SlidesCoeff[CoeffArray,slidesName] take a 64\[Times]N matrix of coefficients (as returned by any of the -Acquire- methods) and returns N pictures (.png by default). The slides are stored in the current directory (type Directory[] to see it and SetDirectory[...] to change it).

Default values:
slideName=''slides''

Possible <Options>: The same as those for the SlidesEvolve function.

EX.1
Options[SlidesCoeff]
CoeffArray = AcquireAll[4 II[x,y,x],Sqrt[2] (II[1,z]+II[2,z]+II[3,z]),2 \[Pi],0,Tsym];
SlidesCoeff[CoeffArray,''SlidesA'',Output\[Rule] ColorQuick,Resolution\[Rule] 250]
";

Options[SlidesCoeff]=Options[SlidesEvolve];
SlidesCoeff[CoeffArray_,slidesName_:"slides",opts:OptionsPattern[]]:=Module[{D,npt,X,L,nbDigit,str},
(*Number of slides*)
D = Dimensions[CoeffArray];
npt= D[[2]];
(*Number of digits to use *)
nbDigit=Length[IntegerDigits[npt]];
X=Transpose[CoeffArray];
For[i=1,i<=npt,i++,
str=ToStr[i,nbDigit,"0"];
L=PlotFromCoeff[X[[i]],OptionValue[basis],FilterRules[{opts},Options[PlotFromCoeff]]];
Export[slidesName <>"_"<>str<>OptionValue[Extension],L,ImageResolution->OptionValue[Resolution]];
];(*For*)
];



SlidesMatrices::usage = "SlidesMatrices[ListMat,slidesName,<Options>] takes a list of N matrices (as returned by the CreateAllMatrices function for instance) and returns N pictures (.png by default). The slides are stored in the current directory (type Directory[] to see it and SetDirectory[...] to change it).

Default values:
slideName=''slides''

Possible <Options>: The same as those for the SlidesEvolve method.

EX.1
Options[SlidesCoeff]
File1 = FileNameJoin[{NotebookDirectory[]<>''/BasicINEPT/''<>''pulse1.txt''}];
File2 = FileNameJoin[{NotebookDirectory[]<>''/BasicINEPT/''<>''pulse2.txt''}];

ps1 = ImportPulseSequence[File1];
ps2 = ImportPulseSequence[File2];
ps3 = {{},0};

InitialDensity  = II[1,x];
Hoffset         = HweakTotal[140,0,0]+HcshiftTotal[30,10,0];
scalingfactor   = 1;
cps             = CombinePulseSequences[ps1,ps2,ps3,Hoffset,scalingfactor];
points          = 1;
AllMatrices     = CreateAllMatrices[InitialDensity,cps,points,report\[Rule] True];

Densities       = AllMatrices[[1]];(*a list with the npt+1 density matrices*)
Propagators     = AllMatrices[[2]];(*a list with the npt+1 propagators*)
EffPropagators  = AllMatrices[[3]];(*a list with the npt+1 effective propagators*)
Hamiltonians    = AllMatrices[[4]];(*a list with the npt+1 actual hamiltonians*)
EffHamiltonians = AllMatrices[[5]];(*a list with the npt+1 effective hamiltonians*)

SlidesMatrices[Densities,''Densities''];
";

Options[SlidesMatrices]=Options[SlidesCoeff];
SlidesMatrices[ListMat_,slidesName_:"slides",opts:OptionsPattern[]]:=Module[{npt,nbDigit,L,str},
(*Number of slides*)
npt= Dimensions[ListMat][[1]];
(*Number of digits to use *)
nbDigit=Length[IntegerDigits[npt]];
(*Export the pictures using with names using the minimum  constant number of digits*)
For[i=1,i<=Length[ListMat],i++,
str=ToStr[i,nbDigit,"0"];
(*Create a list of pictures*)
L=PlotMatrix[ListMat[[i]],FilterRules[{opts},Options[PlotMatrix]]];
Export[slidesName <>"_"<>str<>OptionValue[Extension],L,ImageResolution->OptionValue[Resolution]];
];(*For*)
];


SlidesPulseFile::usage="SlidesPulseFile[ExperimentName] reads pulse files and create slides from the matrices created by the function CrealeAllMatrices.
This method has many options, in particular the user decide which matrices have to be converted into slides.
By default, only the density matrices are converted into slides.
Different folders, for each possible matrices (hamiltonians,efectiveHamiltonians,propagators,effectivePropagators,densityMatrices) are created into a single one labeled ExperimentName.

EX.1
Options[SlidesCoeff]
F1 = FileNameJoin[{NotebookDirectory[]<>''/BasicINEPT/''<>''pulse1.txt''}];
F2 = FileNameJoin[{NotebookDirectory[]<>''/BasicINEPT/''<>''pulse2.txt''}];

InitialDensity  = II[1,x];
SlidesPulseFile[''BasicINEPT'',II[1,z,2,z],file1\[Rule] F1,file2\[Rule]F2,densityMatrices\[Rule] 0,propagators\[Rule] 1]
";
Options[SlidesPulseFile]=Join[{file1-> "",file2->"",file3-> "",
hOffset->Table[0,{i,8},{j,8}] ,scalingfactor-> 1,timepoints-> 1,
basis-> Tsym,
densityMatrices-> 1,propagators-> 0,effectivePropagators-> 0,hamiltonians-> 0,effectiveHamiltonians-> 0,allMatrices-> 0,
directory-> Directory[]},Options[SlidesMatrices]];

SlidesPulseFile[ExperimentName_,InitialDensity_,opts:OptionsPattern[]]:=

Module[{file,pulse,cps,ListDensityMatrices,ListPropagators,ListEffectivePropagators,ListActualHamiltonians,ListEffectiveHamiltonians,initialDir,dir,n,dir1,dir2},

file=Table[OptionValue[Symbol["file"<>ToString[k]]] ,{k,1,3}];
initialDir = Directory[];
(*initialization*)
pulse=Table[{{},0},{k,1,3}];

(* Read *)
Do[If[file[[k]]!=  "",pulse[[k]]=ImportPulseSequence[file[[k]]];],{k,1,3}];
cps=CombinePulseSequences[pulse[[1]],pulse[[2]],pulse[[3]],OptionValue[hOffset],OptionValue[scalingfactor]];

(* Calculation if the matrices *)
{ListDensityMatrices,ListPropagators,ListEffectivePropagators,ListActualHamiltonians,ListEffectiveHamiltonians}=CreateAllMatrices[InitialDensity,cps,OptionValue[timepoints],FilterRules[{opts},Options[CreateAllMatrices]]];
(*Principal Directory: where the other directories containing the slides will be.
Verifies if it exists already and put a number if yes.
*)
dir=OptionValue[directory]<>"/"<>ExperimentName;
dir1=dir;
n=1;
While[FileExistsQ[dir1],dir1=dir<>ToString[n];n++;];

CreateDirectory[dir1];
SetDirectory[dir1];

(* Density matrices *)
If[OptionValue[densityMatrices]==1||OptionValue[allMatrices]==1,
dir2=Directory[]<>"/"<>SymbolName[densityMatrices];
CreateDirectory[dir2];(*Create*)
SetDirectory[dir2];(*Go in*)
(*slides*)
SlidesMatrices[ ListDensityMatrices,"slide",FilterRules[{opts},Options[SlidesMatrices]]];
(*Come back to Experiment directory*)
SetDirectory[dir1];
];

(* Propagators *)
If[OptionValue[propagators]==1||OptionValue[allMatrices]==1,
dir2=Directory[]<>"/"<>SymbolName[propagators];
CreateDirectory[dir2];(*Create*)
SetDirectory[dir2];(*Go in*)
(*slides*)
SlidesMatrices[ ListPropagators,"slide",FilterRules[{opts},Options[SlidesMatrices]]];
(*Come back to Experiment directory*)
SetDirectory[dir1];
];

(* EffectivePropagators *)
If[OptionValue[effectivePropagators]==1||OptionValue[allMatrices]==1,
dir2=Directory[]<>"/"<>SymbolName[effectivePropagators];
CreateDirectory[dir2];(*Create*)
SetDirectory[dir2];(*Go in*)
(*slides*)
SlidesMatrices[ ListEffectivePropagators,"slide",FilterRules[{opts},Options[SlidesMatrices]]];
(*Come back to Experiment directory*)
SetDirectory[dir1];
];

(* ActualHamiltonians *)
If[OptionValue[hamiltonians]==1||OptionValue[allMatrices]==1,
dir2=Directory[]<>"/"<>SymbolName[hamiltonians];
CreateDirectory[dir2];(*Create*)
SetDirectory[dir2];(*Go in*)
(*slides*)
SlidesMatrices[ ListActualHamiltonians,"slide",FilterRules[{opts},Options[SlidesMatrices]]];
(*Come back to Experiment directory*)
SetDirectory[dir1];
];

(* EffectiveHamiltonians *)
If[OptionValue[effectiveHamiltonians]==1||OptionValue[allMatrices]==1,
dir2=Directory[]<>"/"<>SymbolName[effectiveHamiltonians];
CreateDirectory[dir2];(*Create*)
SetDirectory[dir2];(*Go in*)
(*slides*)
SlidesMatrices[ ListEffectiveHamiltonians,"slide",FilterRules[{opts},Options[SlidesMatrices]]];
(*Come back to Experiment directory*)
SetDirectory[dir1];
];

(*Come back to Notebook directory*)
SetDirectory[initialDir];
];
